<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

require dirname(__FILE__).'/../inc/admin/prepend.php';

if (!checkPerm('manage_modules'))
	adminPage::fatalErrorPage(__('You do not have permission to access this page.'));

$action = NULL;
$new_version = false;

$allowed_actions = array('enable','disable','install','uninstall','update','delete','check_updates');
if (!empty($_REQUEST['action']) && in_array(html::escapeHTML($_REQUEST['action']),$allowed_actions))
	$action = html::escapeHTML($_REQUEST['action']);


# Action sur un module...
if (!empty($_REQUEST['module']) && $puntal->modules->moduleExists($_REQUEST['module']))
{
	# Infos module
	$module = $puntal->modules->getModules($_REQUEST['module']);
	
	# "Protection" du module par dfaut
	if ($puntal->modules->isDefaultModule($module['id']) && ($action == 'disable' || $action == 'uninstall'))
		$common_error = __('You can not do this on the default module');

	# Activer ou dsactiver un module
	elseif ($action == 'disable' || $action == 'enable')
	{
		$current_status = constant('pt_mod_'.$module['id']);
		$do = $action == 'enable' ? true : false;
		
		if ($current_status && $do)
			$common_error = sprintf(__('The %s module is already enable'),__($module['name']));
		elseif (!$current_status && !$do)
			$common_error = sprintf(__('The %s module is already disable'),__($module['name']));
		else {
			$pt_settings->setNamespace('puntal');
			$res = $pt_settings->put('pt_mod_'.$module['id'],$do);
			$pt_settings->setNamespace('');
		
			if ($res)
			{
				logIt($action.' module '.$module['id'], 2);
				header('Location: modules.php?updated=1');
				exit;
			}
		}
	}
	
	# Installer un module
	elseif ($action == 'install')
	{
		if ($module['installed'])
			$common_error = sprintf(__('The %s module is already installed'),__($module['name']));
		
		# Demande de confirmation
		elseif (empty($_POST['confirm_install']))
		{
			# Set token
			$puntal->setToken();
			
			# Start confirm page
			adminPage::open(sprintf(__('Install the %s module'),__($module['name'])));
			echo 
			'<h2 class="first"><img src="style/img/ico_module_install.png" alt="" class="icon" /> '.sprintf(__('Install the %s module'),__($module['name'])).'</h2>'."\n".
			'<form action="modules.php" method="post">'."\n".
			'<fieldset><legend>'.__('Please confirm').'</legend>'."\n".
			'<p>'.sprintf(__('Are you sure you want to install module %s?'),__($module['name'])).'</p>'."\n".
			'</fieldset><p>'."\n".
			form::hidden(array('action'), 'install')."\n".
			form::hidden(array('module'), $module['id'])."\n".
			form::hidden(array('confirm_install'), 1)."\n".
			form::hidden(array('ptkn'),$puntal->getToken())."\n".
			'<input class="submit" type="submit" value="'.__('Install').'" /></p></form>';
			adminPage::close();
			exit;
		}
		
		# Installation confirme, alors on installe
		else {
			$puntal->checkToken();
			
			if (file_exists($module['root'].'/inc/install/install.php'))
			{
				require $module['root'].'/inc/install/install.php';
				if (function_exists('install_'.$module['id']))
				{
					# We need a checklist, a file manager and the module installer
					require PT_INC_PATH.'classes/checklist.php';
					require PT_INC_PATH.'libs/files.php';
					require PT_INC_PATH.'classes/module.install.php';
					$install_module = new installModule($puntal->con,$puntal->err,$pt_settings,$module);
					
					call_user_func('install_'.$module['id']);
					
					if ($install_module->check->checkAll())
					{
						logIt('Install module '.__($module['name']), 3);
						adminPage::open(sprintf(__('Module %s successfully installed'),__($module['name'])));
						echo '<h2 class="first"><img src="style/img/ico_module_install.png" alt="" class="icon" /> '.sprintf(__('Module %s successfully installed'),__($module['name'])).'</h2>'."\n";
					}
					else {
						adminPage::open(sprintf(__('Unable to install %s module'),__($module['name'])));
						echo
						'<h2 class="first"><img src="style/img/ico_module_install.png" alt="" class="icon" /> '.sprintf(__('Unable to install %s module'),__($module['name'])).'</h2>'."\n";
					}
					
					echo $install_module->check->getHTML('style/img/check_on.png','style/img/check_off.png','style/img/check_wrn.png');
					echo '<p><a href="modules.php">'.__('Modules').'</a></p>';
					adminPage::close();
					exit;
				}
				else
					$common_error = sprintf(__('The module %s does not have a valid installation process'),__($module['name']));
			}
			else
				$common_error = sprintf(__('The module %s does not have an installation file'),__($module['name']));
		}
	}
	
	# Dsinstaller un module
	elseif ($action == 'uninstall')
	{
		if (!$module['installed'])
			$common_error = sprintf(__('The %s module is already uninstalled'),__($module['name']));
		
		# Demande de confirmation
		elseif (empty($_POST['confirm_uninstall']))
		{
			# Set token
			$puntal->setToken();
			
			# Start confirm page
			adminPage::open(sprintf(__('Uninstall the %s module'),__($module['name'])));
			echo 
			'<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Uninstall the %s module'),__($module['name'])).'</h2>'."\n".
			'<form action="modules.php" method="post">'."\n".
			'<fieldset><legend>'.__('Please confirm').'</legend>'."\n".
			'<p>'.sprintf(__('Are you sure you want to uninstall module %s?'),__($module['name'])).'</p>'."\n".
			'<p>'.__('While doing that you will definitively delete stored datas.').'</p>'."\n".
			'</fieldset><p>'."\n".
			form::hidden(array('action'), 'uninstall')."\n".
			form::hidden(array('module'), $module['id'])."\n".
			form::hidden(array('confirm_uninstall'), 1)."\n".
			form::hidden(array('ptkn'),$puntal->getToken())."\n".
			'<input class="submit" type="submit" value="'.__('Uninstall').'" /></p></form>';
			adminPage::close();
			exit;
		}
		
		# Dsinstallation confirme, alors on dsinstalle
		else {
			$puntal->checkToken();
			
			if (file_exists($module['root'].'/inc/install/uninstall.php'))
			{
				require $module['root'].'/inc/install/uninstall.php';
				if (function_exists('uninstall_'.$module['id']))
				{
					# We need a checklist, a file manager and the module installer
					require PT_INC_PATH.'classes/checklist.php';
					require PT_INC_PATH.'libs/files.php';
					require PT_INC_PATH.'classes/module.install.php';
					$install_module = new installModule($puntal->con,$puntal->err,$pt_settings,$module);
					
					call_user_func('uninstall_'.$module['id']);
					
					if ($install_module->check->checkAll())
					{
						logIt('Uninstall module '.__($module['name']), 3);
						adminPage::open(sprintf(__('Module %s successfully uninstalled'),__($module['name'])));
						echo '<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Module %s successfully uninstalled'),__($module['name'])).'</h2>'."\n";
					}
					else {
						adminPage::open(sprintf(__('Unable to uninstall %s module'),__($module['name'])));
						echo
						'<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Unable to uninstall %s module'),__($module['name'])).'</h2>'."\n";
					}
					
					echo $install_module->check->getHTML('style/img/check_on.png','style/img/check_off.png','style/img/check_wrn.png');
					echo '<p><a href="modules.php">'.__('Modules').'</a></p>';
					adminPage::close();
					exit;
				}
				else
					$common_error = sprintf(__('The module %s does not have a valid uninstallation process'),__($module['name']));
			}
			else
				$common_error = sprintf(__('The module %s does not have an uninstallation file'),__($module['name']));
		}
	}
	
	# Mise  jour d'un module
	elseif ($action == 'update')
	{
		if (file_exists($module['root'].'/inc/install/update.php'))
		{
			require $module['root'].'/inc/install/update.php';
			if (function_exists('update_'.$module['id']))
			{
				# We need a checklist, a file manager and the module installer
				require PT_INC_PATH.'classes/checklist.php';
				require PT_INC_PATH.'libs/files.php';
				require PT_INC_PATH.'classes/module.install.php';
				$install_module = new installModule($puntal->con,$puntal->err,$pt_settings,$module);
				
				call_user_func('update_'.$module['id']);
				
				if ($install_module->check->checkAll())
				{
					logIt('Update module '.__($module['name']), 2);
					adminPage::open(sprintf(__('Module %s successfully updated'),__($module['name'])));
					echo '<h2 class="first"><img src="style/img/ico_module_update.png" alt="" class="icon" /> '.sprintf(__('Module %s successfully updated'),__($module['name'])).'</h2>'."\n";
				}
				else {
					adminPage::open(sprintf(__('Unable to update %s module'),__($module['name'])));
					echo '<h2 class="first"><img src="style/img/ico_module_update.png" alt="" class="icon" /> '.sprintf(__('Unable to update %s module'),__($module['name'])).'</h2>'."\n";
				}
				
				echo $install_module->check->getHTML('style/img/check_on.png','style/img/check_off.png','style/img/check_wrn.png');
				echo '<p><a href="modules.php">'.__('Modules').'</a></p>';
				adminPage::close();
				exit;
			}
			else
				$common_error = sprintf(__('The module %s does not have a valid update process'),__($module['name']));
		}
		else
			$common_error = sprintf(__('The module %s does not have an update file'),__($module['name']));
	}
	
		
	# Supression d'un module
	elseif ($action == 'delete')
	{
		# Demande de confirmation
		if (empty($_POST['confirm_delete']))
		{
			# Set token
			$puntal->setToken();
			
			# Start confirm page
			adminPage::open(sprintf(__('Delete the %s module'),__($module['name'])));
			echo 
			'<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Delete the %s module'),__($module['name'])).'</h2>'."\n".
			'<form action="modules.php" method="post">'."\n".
			'<fieldset><legend>'.__('Please confirm').'</legend>'."\n".
			'<p>'.sprintf(__('Are you sure you want to delete the %s module?'),__($module['name'])).'</p>'."\n".
			'<p>'.__('While doing that you will definitively delete module files.').'</p>'."\n".
			'</fieldset><p>'."\n".
			form::hidden(array('action'), 'delete')."\n".
			form::hidden(array('module'), $module['id'])."\n".
			form::hidden(array('confirm_delete'), 1)."\n".
			form::hidden(array('ptkn'),$puntal->getToken())."\n".
			'<input class="submit" type="submit" value="'.__('Delete').'" /></p></form>';
			adminPage::close();
			exit;
		}
		
		# Supression confirme, alors on supprime
		else {
			$puntal->checkToken();
			
			require PT_INC_PATH.'classes/checklist.php';
			require PT_INC_PATH.'libs/files.php';
			
			# Load installation locales
			l10n::set(dirname(__FILE__).'/../locales/'.$puntal->user['language'].'/install');
			
			$delcheck = new checkList();
			
			# delete module files
			$module_path = path::real(PT_MODULES_PATH).'/'.$module['id'];
			$delcheck->addItem(
				sprintf(__('Delete <em>%s</em> directory'),$module_path),
				files::deltree($module_path),
				sprintf(__('Directory <em>%s</em> successfully deleted'),$module_path),
				sprintf(__('Unable to delete <em>%s</em> directory'),$module_path)
			);
			
			# delete themes files
			$d = dir(PT_THEMES_PATH);
			while (($entry = $d->read()) !== false)
			{
				if ($entry != '.' && $entry != '..' && $entry != '.svn' 
				&& is_dir(PT_THEMES_PATH.$entry) 
				&& file_exists(PT_THEMES_PATH.$entry.'/__infos.php') 
				&& file_exists(PT_THEMES_PATH.'/'.$entry.'/'.$module['id']) 
				&& is_dir(PT_THEMES_PATH.'/'.$entry.'/'.$module['id']))
				{
					$theme_path = path::real(PT_THEMES_PATH).'/'.$entry.'/'.$module['id'];
					$delcheck->addItem(
						sprintf(__('Delete <em>%s</em> directory'),$theme_path),
						files::deltree($theme_path),
						sprintf(__('Directory <em>%s</em> successfully deleted'),$theme_path),
						sprintf(__('Unable to delete <em>%s</em> directory'),$theme_path)
					);
				}
				
				# delete images files if exists
				$img_path = path::real(PT_THEMES_PATH.'img/').'/'.$module['id'];

				if ( file_exists($img_path)) {
					$delcheck->addItem(
						sprintf(__('Delete <em>%s</em> directory'),$img_path),
						files::deltree($img_path),
						sprintf(__('Directory <em>%s</em> successfully deleted'),$img_path),
						sprintf(__('Unable to delete <em>%s</em> directory'),$img_path)
					);
				}
			}
			
			
			if ($delcheck->checkAll())
			{
				logIt('Delete module '.__($module['name']), 3);
				adminPage::open(sprintf(__('Module %s successfully deleted'),__($module['name'])));
				echo '<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Module %s successfully deleted'),__($module['name'])).'</h2>'."\n";
			}
			else {
				adminPage::open(sprintf(__('Unable to delete %s module'),__($module['name'])));
				echo '<h2 class="first"><img src="style/img/ico_module_uninstall.png" alt="" class="icon" /> '.sprintf(__('Unable to delete %s module'),__($module['name'])).'</h2>'."\n";
			}
			
			echo $delcheck->getHTML('style/img/check_on.png','style/img/check_off.png','style/img/check_wrn.png');
			echo '<p><a href="modules.php">'.__('Modules').'</a></p>';
			adminPage::close();
			exit;
		}
	}

}


# Boucle sur la liste des modules
$modules = array();
$modules_updates = array();

foreach ($puntal->modules->getModules() as $m)
{
	# Recherche de mises  jour
	$latest_version = 0;
	$version_info = '<img src="style/img/check_off.png" alt="" class="icon" /> '.__('Unable to check lastest version');
	
	if (!empty($m['url']))
	{
		# Demande explicite par l'utilisateur de recherche de mise  jour
		if ($action == 'check_updates')
			$latest_version = updateCacheLatestRelease($m['url'],$m['id']);
		# Recherche de mise  jour automatique
		elseif (pt_auto_search_updates)
		{
			if (!file_exists(PT_CACHE_PATH.'/latest_versions/'.$m['id']) 
			|| (time()-@filemtime(PT_CACHE_PATH.'/latest_versions/'.$m['id'])) > 86400)
				$latest_version = updateCacheLatestRelease($m['url'],$m['id']);
			else
				$latest_version = trim(@file_get_contents(PT_CACHE_PATH.'/latest_versions/'.$m['id']));
		}
		# fichier cache
		elseif (file_exists(PT_CACHE_PATH.'/latest_versions/'.$m['id']))
			$latest_version = trim(@file_get_contents(PT_CACHE_PATH.'/latest_versions/'.$m['id']));
		
		# Comparaison des versions si on as un numro de dernire version
		if ($latest_version)
		{
			if (compare_version($m['version'],$latest_version) == '-1')
			{
				$version_info = '<li><img src="style/img/check_wrn.png" alt="" class="icon" /> <strong>'.__('A new version of this module is available!').'</strong></li>';
				$new_version = true;
			}
			else
				$version_info = '<li><img src="style/img/check_on.png" alt="" class="icon" /> '.__('You use the last version of this module.').'</li>';
		}
	}
	
	# Module install ? activ ?  jour ?
	$status = '';
	$install = '';
	$update = '';
	$delete = '';

	if ($m['installed'])
	{
		# Status, activ ou dsactiv ?
		if (constant('pt_mod_'.$m['id']))
			$status = '<li><a href="modules.php?action=disable&amp;module='.$m['id'].'" title="'.sprintf(__('Disable the %s module'),__($m['name'])).'" class="disable_module">'.__('Disable').'</a></li>'."\n";
		else
			$status = '<li><a href="modules.php?action=enable&amp;module='.$m['id'].'" title="'.sprintf(__('Enable the %s module'),__($m['name'])).'" class="enable_module">'.__('Enable').'</a></li>'."\n";
		
		# besoin d'tre mis  jour ?
		if ($m['version'] != constant('pt_mod_'.$m['id'].'_version'))
		{
			$update = '<li><a href="modules.php?action=update&amp;module='.$m['id'].'" title="'.sprintf(__('Update the %s module'),__($m['name'])).'" class="upd_module">'.__('Update').'</a></li>'."\n";
			$modules_updates[] = __($m['name']);
		}
		
		# Lien de dsinstallation
		$install = '<li><a href="modules.php?action=uninstall&amp;module='.$m['id'].'" title="'.sprintf(__('Uninstall the %s module'),__($m['name'])).'" class="uninstall_module">'.__('Uninstall').'</a></li>'."\n";
	}
	else {
		# Lien d'installation
		$install = '<li><a href="modules.php?action=install&amp;module='.$m['id'].'" title="'.sprintf(__('Install the %s module'),__($m['name'])).'" class="install_module">'.__('Install').'</a></li>'."\n";
		
		$delete = '<li><a href="modules.php?action=delete&amp;module='.$m['id'].'" title="'.sprintf(__('Delete the %s module'),__($m['name'])).'" class="delete_module">'.__('Delete').'</a></li>'."\n";
	}
	
	$modules[] = array(
		'id' => $m['id'],
		'name' => __($m['name']),
		'desc' => __($m['desc']),
		'author' => $m['author'],
		'version' => $m['version'],
		'status' => $status,
		'install' => $install,
		'update' => $update,
		'delete' => $delete,
		'release' => $version_info
	);
}

if ($action == 'check_updates')
{
	if ($new_version)
		$common_warning = __('There are new versions available!');
	else
		$common_message = __('You have the last versions for the checked module.');
}

if (!empty($modules_updates))
	$common_warning = sprintf(__('The following module(s) must be updated: %s'), implode(', ', $modules_updates));

# message de confirmation
if (!empty($_GET['updated']))
	$common_message = __('Configuration updated');

#Affichage 
adminPage::open(__('Modules'));

?>
<h2 class="first"><img src="style/img/ico_modules.png" alt="" class="icon" /> <?php echo __('Modules'); ?></h2>
	
	<p><a href="modules.php?action=check_updates" class="imglink updates_link"><?php echo __('Check for updates.') ?></a></p>

	<?php foreach($modules as $m) : ?>
	
	<div class="module">
		<h3><?php
		echo $m['name'];
		if ($puntal->modules->isDefaultModule($m['id'])) 
			echo ' <img src="style/img/ico_star.png" alt="('.__('Default module').')" />';
		?></h3>
		<ul>
			<li><?php echo __($m['desc']) ?></li>
			<li><?php echo __('Author'). ' : '.$m['author']; ?></li>
			<li><?php echo __('Release'). ' : '.$m['version']; ?></li>
			<?php echo $m['release'].$m['status'].$m['update'].$m['install'].$m['delete']; ?>
		</ul>
	</div>
	
	<?php endforeach; ?>

<?php 
adminPage::close(); 
?>