<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

require dirname(__FILE__).'/../inc/admin/prepend.php';

if (!checkPerm('puntal_config'))
	adminPage::fatalErrorPage(__('You do not have permission to access this page.'));

$p_title = pt_portal_title;
$p_desc = pt_portal_desc;
$p_default_module = pt_default_module;
$p_announcement = pt_announcement;
$p_additional_navlinks = pt_additional_navlinks;
$p_path = pt_portal_path;
$p_url = pt_portal_url;
$p_forum_url = pt_forum_url;
$p_url_scan = pt_url_scan;
$p_date_format = pt_date_format;
$p_time_format = pt_time_format;
$p_auto_search_updates = pt_auto_search_updates;
$p_send_real_xml = pt_send_real_xml;
$p_cache_ttl = pt_puntal_cache_ttl;

# Liste des modules
$p_enabled_mod_array = array();
$page_modules_array = array();
$modules_array = array();
foreach ($puntal->modules->getInstalledModules() as $m)
{
	if (constant('pt_mod_'.$m['id']))
		$p_enabled_mod_array[] = $m['id'];
	
	if ($m['page_module'])
		$page_modules_array[__($m['name'])] = $m['id'];
	
	$modules_array[__($m['name'])] = $m['id'];
}

# Modification de la configuration
if (!empty($_POST['form_sent']) && $puntal->checkToken())
{	
	$p_title = trim($_POST['p_title']);
	$p_desc = trim($_POST['p_desc']);
	$p_default_module = trim($_POST['p_default_module']);
	$p_announcement = trim($_POST['p_announcement']);
	$p_additional_navlinks = trim($_POST['p_additional_navlinks']);
	$p_path = trim($_POST['p_path']);
	$p_url = trim($_POST['p_url']);
	$p_forum_url = trim($_POST['p_forum_url']);
	$p_url_scan = trim($_POST['p_url_scan']);
	$p_date_format = trim($_POST['p_date_format']);
	$p_time_format = trim($_POST['p_time_format']);	
	$p_auto_search_updates = trim($_POST['p_auto_search_updates']);
	$p_send_real_xml = trim($_POST['p_send_real_xml']);
	$p_cache_ttl = trim($_POST['p_cache_ttl']);
		
	$p_enabled_mod_array = !empty($_POST['p_mods']) ? $_POST['p_mods'] : array();
		
	# Titre portail requis
	if ($p_title == '')
		$puntal->err->setError(__('You must give a title for your portal.'));
	
	# Chemin portail requis
	if ($p_path == '')
		$puntal->err->setError(__('You must give an application path.'));
	
	# URL portail requise
	if ($p_url == '')
		$puntal->err->setError(__('You must give a portal <acronym title="Uniform Resource Locator">URL</acronym>.'));
	
	# URL forums requise
	if ($p_forum_url == '')
		$puntal->err->setError(__('You must give a forums <acronym title="Uniform Resource Locator">URL</acronym>.'));
	
	# Items de menu additionnels
	if ($p_additional_navlinks != '')
		$p_additional_navlinks = $puntal->linebreaks($p_additional_navlinks);
	
	# Mise ? jour de la configuration
	if (!$puntal->err->hasError())
	{
		# Correction chemin Puntal
		$p_path = preg_replace('|^/+|','',$p_path);
		$p_path = '/'.$p_path;
		
		# Correction URL Puntal
		$p_url = preg_replace('|^/+|','',$p_url);
		$p_url = '/'.$p_url;
		
		# Correction URL forums
		$p_forum_url = str_replace('index.php','',$p_forum_url);
		$p_forum_url = preg_replace('|/+$|','',$p_forum_url);
		$p_forum_url = preg_replace('|^/+|','',$p_forum_url);
		$p_forum_url = '/'.$p_forum_url.'/';
		
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		$res &= $pt_settings->put('pt_portal_title',$p_title);
		$res &= $pt_settings->put('pt_portal_desc',$p_desc);
		$res &= $pt_settings->put('pt_date_format', $p_date_format);
		$res &= $pt_settings->put('pt_time_format', $p_time_format);
		$res &= $pt_settings->put('pt_default_module',$p_default_module);
		$res &= $pt_settings->put('pt_announcement',$p_announcement);
		$res &= $pt_settings->put('pt_additional_navlinks',$p_additional_navlinks);
		$res &= $pt_settings->put('pt_portal_path',$p_path);
		$res &= $pt_settings->put('pt_portal_url',$p_url);
		$res &= $pt_settings->put('pt_forum_url',$p_forum_url);
		$res &= $pt_settings->put('pt_url_scan',$p_url_scan);
		$res &= $pt_settings->put('pt_auto_search_updates',$p_auto_search_updates);
		$res &= $pt_settings->put('pt_send_real_xml',$p_send_real_xml);
		$res &= $pt_settings->put('pt_puntal_cache_ttl',$p_cache_ttl);
		
		foreach ($puntal->modules->getInstalledModules() as $m)
		{
			if (in_array($m['id'], $p_enabled_mod_array))
				$res &= $pt_settings->put('pt_mod_'.$m['id'],true);
			else
				$res &= $pt_settings->put('pt_mod_'.$m['id'],false);
		}
		
		$pt_settings->setNamespace('');
	
		if ($res)
		{
			logIt('Edit Puntal configuration', 2);
			header('Location: settings.php?updated=1');
			exit;
		}
	}
}

# Cache ttl array
$cache_ttl_array = array(
	__('30 minutes') => 30,
	__('1 hour') => 60,
	__('2 hours') => 120,
	__('6 hours') => 360,
	__('12 hours') => 720,
	__('1 day') => 1440
);
# Set token
$puntal->setToken();

# Message de confirmation
if (!empty($_GET['updated']))
	$common_message = __('Configuration updated');

# Affichage
adminPage::open(__('Puntal configuration'),	
'<script type="text/javascript">'."\n".
"//<![CDATA[\n".
"$(function() {
	$('.checkboxes-helpers').each(function() {
		puntal.checkboxesHelpers(this);
	});
});".
"//]]>\n".
"</script>"."\n".
adminPage::jsPageTabs("puntal-config")
);
?>

<h2 class="first"><img src="style/img/ico_config.png" alt="" class="icon" /> <?php echo __('Puntal configuration'); ?></h2>

<form method="post" action="settings.php" id="form-puntal-settings">	
				
	<div id="puntal-config" class="multi-part" title="<?php echo __('Puntal configuration'); ?>">
	<fieldset>
		<legend><?php echo __('Puntal configuration'); ?></legend>	
		
		<p class="field"><label for="p_title"><?php echo __('Portal title') ?></label>
		<?php echo form::field('p_title', '50', '255', html::escapeHTML($p_title)) ?>
		<span class="desc"><?php echo __('The title of the portal. This field <strong>cannot</strong> contain <acronym title="Hypertext Markup Language">HTML</acronym>.'); ?></span></p>
	
		<p class="field"><label for="p_desc"><?php echo __('Portal description') ?></label>
		<?php echo form::field('p_desc', '50', '255', html::escapeHTML($p_desc)) ?>
		<span class="desc"><?php echo __('A short description of portal. This field can contain <acronym title="Hypertext Markup Language">HTML</acronym>.'); ?></span></p>
		
		<p><span class="fakelabel"><?php echo __('Modules') ?></span>
		<span class="desc"><?php echo __('Activate here the modules which you wish to use.') ?></span></p>
		<ul class="checklist">
		<?php
		foreach ($modules_array as $m_name=>$m_id)
			echo "\t\t".'<li><label>'.form::checkbox(array('p_mods[]'), $m_id, in_array($m_id,$p_enabled_mod_array), 'checkbox').$m_name.'</label></li>'."\n";
		?>
		</ul>
		<p><span class="desc checkboxes-helpers"></span></p>
		
		<p class="field"><label for="p_default_module"><?php echo __('Default module') ?></label>
		<?php echo form::combo('p_default_module', $page_modules_array, $p_default_module) ?>
		<span class="desc"><?php echo __('Choose the module which will be display by default on the portal.'); ?></span></p>
				
		<p><span class="fakelabel"><?php echo __('Forums anoucement'); ?></span>
		<label><?php echo form::radio(array('p_announcement'), 1, $p_announcement, 'radio').__('Yes') ?></label>
		<label><?php echo form::radio(array('p_announcement'), 0, !$p_announcement, 'radio').__('No') ?></label>
		<span class="desc"><?php echo __('Display forums anoucement on the gate.') ?></span></p>
		
		<p class="field"><label for="p_additional_navlinks"><?php echo __('Additional menu items') ?></label>
		<?php echo form::textArea('p_additional_navlinks', '55', '3', html::escapeHTML($p_additional_navlinks)) ?>
		<span class="desc"><?php echo __('By entering HTML hyperlinks into this textbox, any number of items can be added to the navigation menu at the top of all pages. The format for adding new links is X = &lt;a href="URL"&gt;LINK&lt;/a&gt; where X is the position at which the link should be inserted (e.g. 0 to insert at the beginning and 2 to insert after "User list"). Separate entries with a linebreak.') ?></span></p>
		
	</fieldset>
	</div>
	
	<div id="path-url" class="multi-part" title="<?php echo __('Path and URLs'); ?>">
	<fieldset>
		<legend><?php echo __('Path and <acronym title="Uniform Resource Locator">URL</acronym>s'); ?></legend>
		
		<p class="field"><label for="p_path"><?php echo __('Application path') ?></label>
		<?php echo form::field('p_path', '50', '255', html::escapeHTML($p_path)) ?>
		<span class="desc"><?php printf(__('The path of the Puntal directory from %s<br />If you put Puntal at the root of your website, give a simple <code>/</code><br />If you put Puntal in a directory, for example a directory at the root of your website named <code>puntal</code>, give <code>/puntal/</code>'), puntal::getHost()) ?></span></p>
		
		<p class="field"><label for="p_url"><?php echo __('Portal <acronym title="Uniform Resource Locator">URL</acronym>') ?></label>
		<?php echo form::field('p_url', '50', '255', html::escapeHTML($p_url)) ?>
		<span class="desc"><?php printf(__('The <acronym title="Uniform Resource Locator">URL</acronym> of the portal from %s'), puntal::getHost()) ?></span></p>
		
		<p class="field"><label for="p_forum_url"><?php echo __('Forums <acronym title="Uniform Resource Locator">URL</acronym>') ?></label>
		<?php echo form::field('p_forum_url', '50', '255', html::escapeHTML($p_forum_url)) ?>
		<span class="desc"><?php printf(__('The <acronym title="Uniform Resource Locator">URL</acronym> of the forums from %s<br /><strong>Caution!</strong> Be sure to provide exact information, without what you will have errors.'), puntal::getHost()) ?></span></p>
		
		<p><span class="fakelabel"><?php echo __('<acronym title="Uniform Resource Locator">URL</acronym>s type') ?></span>
		<label><?php echo form::radio(array('p_url_scan'), 'path_info', ($p_url_scan == 'path_info'), 'radio') ?>Path info</label>
		<label><?php echo form::radio(array('p_url_scan'), 'query_string', ($p_url_scan == 'query_string'), 'radio') ?>Query string</label>
		<span class="desc"><?php echo __('The type of <acronym title="Uniform Resource Locator">URL</acronym>s defines the form of <acronym title="Uniform Resource Locator">URL</acronym>s (links) in your portal.') ?></span></p>
		<ul class="desc">
			<li><?php echo __('Query string: yours links will have the <code>http://yourwebsite/puntal/index.php?module/</code> form. This method will function on all the servers. The <acronym title="Uniform Resource Locator">URL</acronym> of the portal must have <code>/index.php?</code> form.') ?></li>
			<li><?php echo __('Path info: yours links will have the <code>http://yourwebsite/puntal/index.php/module/</code> form. This type allows a better referencing and sometimes to simplify <acronym title="Uniform Resource Locator">URL</acronym>s such as for example <code>http://yourwebsite/puntal/module/</code> But this type does not function everywhere. Do not forget either to change the <acronym title="Uniform Resource Locator">URL</acronym> of the portal by adding a / at the end. Thus, the <acronym title="Uniform Resource Locator">URL</acronym> of the portal must have <code>/index.php/</code> form.') ?></li>
		</ul>
		
	</fieldset>
	</div>
	
	<div id="adv-config" class="multi-part" title="<?php echo __('Advanced configuration'); ?>">
	<fieldset>
		<legend><?php echo __('Advanced configuration'); ?></legend>
		
		<p class="field"><label for="p_date_format"><?php echo __('Date format') ?></label>
		<?php echo form::field('p_date_format',40,'',html::escapeHTML($p_date_format)) ?>
		<span class="desc"><?php printf(__('[Current Format: %s] See <a href="http://www.php.net/manual/en/function.strftime.php">here</a> for the options of formatting.'), dt::str($p_date_format)) ?></span></p>
		
		<p class="field"><label for="p_time_format"><?php echo __('Time format') ?></label>
		<?php echo form::field('p_time_format',40,'',html::escapeHTML($p_time_format)) ?>
		<span class="desc"><?php printf(__('[Current Format: %s] See <a href="http://www.php.net/manual/en/function.strftime.php">here</a> for the options of formatting.'), dt::str($p_time_format)) ?></span></p>
		
		<p><span class="fakelabel"><?php echo __('Automatic search for updates') ?></span>
		<label><?php echo form::radio(array('p_auto_search_updates'), 1, $p_auto_search_updates, 'radio').__('Yes') ?></label>
		<label><?php echo form::radio(array('p_auto_search_updates'), 0, !$p_auto_search_updates, 'radio').__('No') ?></label>
		<span class="desc"><?php echo __('If enable Puntal will automatically search for updates.') ?></span></p>
		
		<p><span class="fakelabel"><?php echo __('Content-Type: application/xhtml+xml') ?></span>
		<label><?php echo form::radio(array('p_send_real_xml'), 1, $p_send_real_xml, 'radio').__('Yes') ?></label>
		<label><?php echo form::radio(array('p_send_real_xml'), 0, !$p_send_real_xml, 'radio').__('No') ?></label>
		<span class="desc"><?php echo __('Send Content-Type: application/xhtml+xml if possible? Caution: do not support any error of XHTML syntax. Leave to no if you do not know.') ?></span></p>

		<p class="field"><label for="p_cache_ttl"><?php echo __('Cache time to live') ?></label>
		<?php echo form::combo('p_cache_ttl', $cache_ttl_array, $p_cache_ttl) ?>
		<span class="desc"><?php echo __('Changes the amount of time of puntal caching system before expiring.') ?></span></p>

		
	</fieldset>
	</div>
	
	<p><?php echo 
	form::hidden(array('form_sent'),1)."\n".
	form::hidden(array('ptkn'),$puntal->getToken()); ?>
	<input class="submit" type="submit" value=" <?php echo __('Save') ?> " /></p>
		
</form>

<?php adminPage::close(); ?>
