<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

/**
	
	@brief puntalUpgrade module class
	
        <strong>FR</strong> Classe charge de la mise  jour automatique de puntal<br/>
        <strong>EN</strong> Class in charge of the automatic update of puntal
*/
class puntalUpgrade
{

   /**
        perform<br/>

        <strong>FR</strong> Fonction charge de la mise  jour automatique de puntal
        <strong>EN</strong> Function in charge of the automatic update of puntal
        */
	function perform()
	{
		$res = true;
		
		# mise ? jour 2.0 beta 3
		if (compare_version(pt_version,'2.0.beta.3') == '-1')
			$res &= puntalUpgrade::up_to_2_0_beta_3();
						
		# mise ? jour 2.0 rc 1
		if (compare_version(pt_version,'2.0.RC.1') == '-1')
			$res &= puntalUpgrade::up_to_2_0_rc_1();
						
		# mise ? jour 2.0 rc 2
		if (compare_version(pt_version,'2.0.RC.2') == '-1')
			$res &= puntalUpgrade::up_to_2_0_rc_2();

		# mise ? jour 2.0.0
		if (compare_version(pt_version,'2.0.0') == '-1') 
			$res &= puntalUpgrade::up_to_2_0();
			
		# mise ? jour 2.1.0
		if (compare_version(pt_version,'2.1.0') == '-1') 
			$res &= puntalUpgrade::up_to_2_1_0();
		
				
		return $res;
	}
	

   /**
        up_to_2_0_beta_3<br/>

        <strong>FR</strong>  mise  jour 2.0 beta 3
        <strong>EN</strong> update to 2.0 beta 3
        */    
	function up_to_2_0_beta_3()
	{
		global $pt_settings;
		
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		$res &= $pt_settings->put('pt_version', '2.0 beta 3');
		
		$pt_settings->setNamespace('');
		
		return $res;
	}
	
   /**
        up_to_2_0_rc_1<br/>

        <strong>FR</strong>  mise  jour 2.0rc1 - cette mise  jour concerne l'ajout de parametre de formatage
        <strong>EN</strong> update to 2.0 rc1 - this release concern the adding of formatting parameters
        */    
	function up_to_2_0_rc_1()
	{
		global $pt_settings;
		
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		
		$res &= $pt_settings->put('pt_date_format', '%A %e %B %Y', 'string');
		$res &= $pt_settings->put('pt_time_format', '%H:%M', 'string');
		$res &= $pt_settings->put('pt_auto_search_updates', false, 'boolean');
		
		$res &= $pt_settings->put('pt_version', '2.0 RC 1');
		
		$pt_settings->setNamespace('');
		
		return $res;
	}
	
	
   /**
        up_to_2_0_rc_2<br/>

        <strong>FR</strong>  mise  jour 2.0rc2  - cette mise  jour concerne l'ajout du filtre html
        <strong>EN</strong> update to 2.0 rc2  - this release concern the html filter
        */    
	function up_to_2_0_rc_2()
	{
		global $pt_settings;
		
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		
		$res &= $pt_settings->put('pt_html_filter', true, 'boolean');
		
		$res &= $pt_settings->put('pt_version', '2.0 RC 2');
		
		$pt_settings->setNamespace('');
		
		return $res;
	}	
	
   /**
        up_to_2_0<br/>

        <strong>FR</strong>  mise  jour 2.0 - cette mise  jour concerne l'integration de la puntoolbar et supprime le filtre html
        <strong>EN</strong> update to 2.0 - this release concerne the integration of the puntoolbar and drop the html filter
        */    
	function up_to_2_0()
	{
		global $pt_settings,$puntal;
				
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		
		#puntoolbar
		$res &= $pt_settings->put('pt_ptb_img_pack','smooth', 'string');
		$res &= $pt_settings->put('pt_ptb',1, 'integer');
		$res &= $pt_settings->put('pt_ptb_nb_smilies',12,'integer');
		$res &= $pt_settings->put('pt_ptb_smilies_pop_up_width',120,'integer');
		$res &= $pt_settings->put('pt_ptb_smilies_pop_up_height',150,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_plus',1,'integer');		
		$res &= $pt_settings->put('pt_ptb_bt_min',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_left',1,'integer');		
		$res &= $pt_settings->put('pt_ptb_bt_right',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_center',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_justify',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_sup',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_sub',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_b',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_i',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_u',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_s',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_q',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_c',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_acr',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_col',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_bq',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_pre',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_hr',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_link',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_img',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_email',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_nospam',1,'integer');
		$res &= $pt_settings->put('pt_ptb_bt_smilies',1,'integer');
		
	
		$res &= $pt_settings->put('pt_version', '2.0.0');
		
		$res &= $pt_settings->drop('pt_html_filter');
					
		$pt_settings->setNamespace('');

		$query = 'CREATE TABLE '.DB_PREFIX."pt_smilies (
					id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
					image VARCHAR(60) NOT NULL DEFAULT '',
					text VARCHAR(60) NOT NULL DEFAULT '',
					disp_position INT(10) NOT NULL DEFAULT 0,
					PRIMARY KEY (id)
					) TYPE=MyISAM;";
					
		$puntal->con->execute($query) or $puntal->err->fatalError($puntal->con->error());

		$smiley_text = array(':)', '=)', ':|', '=|', ':(', '=(', ':D', '=D', ':o', ':O', ';)', ':/', ':P', ':lol:', ':mad:', ':rolleyes:', ':cool:');
		$smiley_img = array('smile.png', 'smile.png', 'neutral.png', 'neutral.png', 'sad.png', 'sad.png', 'big_smile.png', 'big_smile.png', 'yikes.png', 'yikes.png', 'wink.png', 'hmm.png', 'tongue.png', 'lol.png', 'mad.png', 'roll.png', 'cool.png');

		$num_smilies = count($smiley_text);
		for ($i = 0; $i < $num_smilies; ++$i) {
			$query = 'INSERT INTO '.DB_PREFIX.'pt_smilies (image, text, disp_position)  VALUES(\''.$smiley_img[$i].'\', \''.$puntal->con->escapeStr($smiley_text[$i]).'\', '.$i.')';
			$puntal->con->execute($query) or $puntal->err->fatalError($puntal->con->error());
		}
					
		return $res;
		
	}

   /**
        up_to_2_1_0<br/>

        <strong>FR</strong>  mise  jour 2.1.0 - cette mise  jour concerne l'instauration d'un cache de puntal  60minutes
        <strong>EN</strong> update to 2.1.0 - this release concerne the usage of the puntal cache fixed to 60minutes
        */    
	function up_to_2_1_0()	
    {
		global $pt_settings,$puntal;
		
				
		$pt_settings->setNamespace('puntal');
		
		$res = true;
		$res &= $pt_settings->put('pt_puntal_cache_ttl',60,'integer');
		$res &= $pt_settings->put('pt_version', '2.1.0');
		$pt_settings->setNamespace('');
		
		return $res;
	}
}

?>
