<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

class puntal
{
	var $con;
	var $err;
	var $pun_config;
	var $user;
	var $url;
	var $bans;
	var $modules;
	var $behaviors;
	var $ses;

	/**
	@function puntal
	
	"Constructeur". Cette m�thode initialise l'objet puntal.

	*/
	function puntal(&$con,&$error)
	{
		$this->con = $con;
		$this->err = $error;
		$this->user = array();
		$this->url = array();
		$this->behaviors = array();
				
		# Init modules
		$this->modules = new modules();
				
		# Init blocks
		$this->blocks = new blocks();
		
		# Init cookie session
		$this->ses = new cookieSession('PUNTAL_TOKEN');
			
			
		$pun_config = '';
		# Load cached config
		if (file_exists(PUN_ROOT.'cache/cache_config.php'))
			require PUN_ROOT.'cache/cache_config.php';
		
		if (!defined('PUN_CONFIG_LOADED') && $this->_generate_config_cache())
			require PUN_ROOT.'cache/cache_config.php';
		
		$this->pun_config =& $pun_config;
		
		# Current user infos
		$this->_check_cookie();
		
		$this->user['is_admin'] = ($this->user['g_id'] == PUN_ADMIN) ? true : false;
		$this->user['is_mod'] 	= ($this->user['g_id'] == PUN_MOD) 	 ? true : false;
		$this->user['is_admod'] = ($this->user['g_id'] < PUN_GUEST)  ? true : false;

		# Load cached bans
		if (file_exists(PUN_ROOT.'cache/cache_bans.php'))
			require PUN_ROOT.'cache/cache_bans.php';
			
		$pun_bans = '';
		if (!defined('PUN_BANS_LOADED') && $this->_generate_bans_cache())
			require PUN_ROOT.'cache/cache_bans.php';
		
		$this->bans =& $pun_bans;
		
		# Check if current user is banned
		$this->_check_bans();
		
		# Update online list
		$this->_update_users_online();
	}
	
	/**
	@function registerURL
	
	D�finit les diff�rents mod�les d'URL.
	
	@param	string	type	Type d'URL
	@param	string	url		Mod�le d'URL
	*/
	function registerURL($type,$url)
	{
		$this->url[$type] = $url;
	}
	
	/**
	@function url
	
	Retourne un mod�le d'URL.
	
	@param	string	type	Type d'URL
	*/
	function url($type)
	{
		if (isset($this->url[$type]))
			return $this->url[$type];
		
		return false;
	}
	
	
	/* ===================================================
	Common authentification methods
	=================================================== */

	/**
	@function _check_cookie
	@author Rickard Andersson from PunBB project
	
	@return	
	*/	
	function _check_cookie()
	{
		global $cookie_name;
		
		$now = time();
		$expire = $now + 31536000;	// The cookie expires after a year
	
		// We assume it's a guest
		$cookie = array('user_id' => 1, 'password_hash' => 'Invit�');
		
		// If a cookie is set, we get the user_id and password hash from it
		if (isset($_COOKIE[$cookie_name]))
			list($cookie['user_id'], $cookie['password_hash']) = @unserialize($_COOKIE[$cookie_name]);
		
		if ($cookie['user_id'] > 1)
		{
			// Check if there's a user with the user ID and password hash from the cookie
			$result = $this->con->select('SELECT u.*, g.*, o.logged, o.idle FROM '.DB_PREFIX.'users AS u INNER JOIN '.DB_PREFIX.'groups AS g ON u.group_id=g.g_id LEFT JOIN '.DB_PREFIX.'online AS o ON o.user_id=u.id WHERE u.id='.intval($cookie['user_id'])) or $this->err->fatalError($this->con->error());
			
			$this->user = $result->getData('0');
			
			// If user authorisation failed
			if (!isset($this->user['id']) || md5($GLOBALS['cookie_seed'].$this->user['password']) !== $cookie['password_hash'])
			{
				$this->_pun_setcookie(0, $this->_random_pass(8), $expire);
				$this->_set_default_user();
	
				return false;
			}
				
			// Set a default style if the user selected style no longer exists
			if (!@file_exists(PUN_ROOT.'style/'.$this->user['style'].'.css'))
				$this->user['style'] = $this->pun_config['o_default_style'];
	
			if (!$this->user['disp_topics'])
				$this->user['disp_topics'] = $this->pun_config['o_disp_topics_default'];
			if (!$this->user['disp_posts'])
				$this->user['disp_posts'] = $this->pun_config['o_disp_posts_default'];
	
			if ($this->user['save_pass'] == '0')
				$expire = 0;
	
			// Define this if you want this visit to affect the online list and the users last visit data
			if (!defined('PUN_QUIET_VISIT'))
			{
				// Update the online list
				if (!$this->user['logged'])
				{
					$this->con->execute('INSERT INTO '.DB_PREFIX.'online (user_id, ident, logged) VALUES('.$this->user['id'].', \''.$this->con->escapeStr($this->user['username']).'\', '.$now.')') or $this->err->fatalError($this->con->error());
				}
				else {
					// Special case: We've timed out, but no other user has browsed the forums since we timed out
					if ($this->user['logged'] < ($now-$this->pun_config['o_timeout_visit']))
					{
						$this->con->execute('UPDATE '.DB_PREFIX.'users SET last_visit='.$this->user['logged'].' WHERE id='.$this->user['id']) or $this->err->fatalError($this->con->error());
						$this->user['last_visit'] = $this->user['logged'];
					}
	
					$idle_sql = ($this->user['idle'] == '1') ? ', idle=0' : '';
					
					$this->con->execute('UPDATE '.DB_PREFIX.'online SET logged='.$now.$idle_sql.' WHERE user_id='.$this->user['id']) or $this->err->fatalError($this->con->error());
				}
			}
	
			$this->user['is_guest'] = false;
		}
		else
			$this->_set_default_user();
	}

	/**
	@function _set_default_user
	@author Rickard Andersson from PunBB project
	
	Fill $this->user with default values (for guests)
	
	@return	
	*/	
	function _set_default_user()
	{
		$remote_addr = (isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '127.0.0.1');
	
		// Fetch guest user
		$result = $this->con->select('SELECT u.*, g.*, o.logged FROM '.DB_PREFIX.'users AS u INNER JOIN '.DB_PREFIX.'groups AS g ON u.group_id=g.g_id LEFT JOIN '.DB_PREFIX.'online AS o ON o.ident=\''.$remote_addr.'\' WHERE u.id=1') or $this->err->fatalError($this->con->error());
		
		if ($result->isEmpty())
			exit('Impossible de retrouver les informations invit�. La table \''.DB_PREFIX.'users\' doit contenir une entr�e avec un id = 1 qui repr�sente les utilisateurs anonymes.');
		
		$this->user = $result->getData('0');
		
		// Update online list
		if (!$this->user['logged'])
		{
			$this->con->execute('INSERT INTO '.DB_PREFIX.'online (user_id, ident, logged) VALUES(1, \''.$this->con->escapeStr($remote_addr).'\', '.time().')') or $this->err->fatalError($this->con->error());
		}
		else {
			$this->con->execute('UPDATE '.DB_PREFIX.'online SET logged='.time().' WHERE ident=\''.$this->con->escapeStr($remote_addr).'\'') or $this->err->fatalError($this->con->error());
		}
	
		$this->user['disp_topics'] = $this->pun_config['o_disp_topics_default'];
		$this->user['disp_posts'] = $this->pun_config['o_disp_posts_default'];
		$this->user['timezone'] = $this->pun_config['o_server_timezone'];
		$this->user['language'] = $this->pun_config['o_default_lang'];
		$this->user['style'] = $this->pun_config['o_default_style'];
		$this->user['is_guest'] = true;
	}

	/**
	@function _pun_setcookie
	@author Rickard Andersson from PunBB project
	
	Set a cookie, PunBB style!
	
	@return	
	*/	
	function _pun_setcookie($user_id, $password_hash, $expire)
	{
		// Enable sending of a P3P header by removing // from the following line (try this if login is failing in IE6)
	//	@header('P3P: CP="CUR ADM"');
	
		setcookie($GLOBALS['cookie_name'], serialize(array($user_id, md5($GLOBALS['cookie_seed'].$password_hash))), $expire, $GLOBALS['cookie_path'], $GLOBALS['cookie_domain'], $GLOBALS['cookie_secure']);
	}

	/**
	@function _random_pass
	@author Rickard Andersson from PunBB project
	
	Generate a random password of length $len
	
	@return	string
	*/	
	function _random_pass($len)
	{
		$chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
	
		$password = '';
		for ($i = 0; $i < $len; ++$i)
			$password .= substr($chars, (mt_rand() % strlen($chars)), 1);
	
		return $password;
	}

	/**
	@function _pun_hash
	@author Rickard Andersson from PunBB project
	
	Compute a hash of $str
	Uses sha1() if available. If not, SHA1 through mhash() if available. If not, fall back on md5().
	
	@return	string
	*/	
	function _pun_hash($str)
	{
		if (function_exists('sha1'))	// Only in PHP 4.3.0+
			return sha1($str);
		else if (function_exists('mhash'))	// Only if Mhash library is loaded
			return bin2hex(mhash(MHASH_SHA1, $str));
		else
			return md5($str);
	}

	/**
	@function _check_bans
	@author Rickard Andersson from PunBB project
	
	Check whether the connecting user is banned (and delete any expired bans while we're at it)
	
	@return	
	*/	
	function _check_bans()
	{
		// Admins aren't affected
		if ($this->user['g_id'] == PUN_ADMIN || !$this->bans)
			return;
	
		// Add a dot at the end of the IP address to prevent banned address 192.168.0.5 from matching e.g. 192.168.0.50
		$user_ip = (isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '127.0.0.1').'.';
		$bans_altered = false;
	
		foreach ($this->bans as $cur_ban)
		{
			// Has this ban expired?
			if ($cur_ban['expire'] != '' && $cur_ban['expire'] <= time())
			{
				$this->con->execute('DELETE FROM '.DB_PREFIX.'bans WHERE id='.$cur_ban['id']) or $this->err->fatalError($this->con->error());
				$bans_altered = true;
				continue;
			}
	
			if ($cur_ban['username'] != '' && !strcasecmp($this->user['username'], $cur_ban['username']))
			{
				$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE ident=\''.$this->con->escapeStr($this->user['username']).'\'') or $this->err->fatalError($this->con->error());
				header('Location: '.pt_portal_url);
			}
	
			if ($cur_ban['ip'] != '')
			{
				$cur_ban_ips = explode(' ', $cur_ban['ip']);
	
				for ($i = 0; $i < count($cur_ban_ips); ++$i)
				{
					$cur_ban_ips[$i] = $cur_ban_ips[$i].'.';
	
					if (substr($user_ip, 0, strlen($cur_ban_ips[$i])) == $cur_ban_ips[$i])
					{
						$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE ident=\''.$this->con->escapeStr($this->user['username']).'\'') or $this->err->fatalError($this->con->error());
						header('Location: '.pt_portal_url);
					}
				}
			}
		}
	
		// If we removed any expired bans during our run-through, we need to regenerate the bans cache
		if ($bans_altered) $this->_generate_bans_cache();
	}

	/**
	@function _update_users_online
	@author Rickard Andersson from PunBB project
	
	Update "Users online"
	
	@return	
	*/	
	function _update_users_online()
	{
		$now = time();
	
		// Fetch all online list entries that are older than "o_timeout_online"
		$cur_user = $this->con->select('SELECT * FROM '.DB_PREFIX.'online WHERE logged<'.($now-$this->pun_config['o_timeout_online'])) or $this->err->fatalError($this->con->error());

		while ($cur_user->fetch())
		{
			// If the entry is a guest, delete it
			if ($cur_user->f('user_id') == '1')
			{
				$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE ident=\''.$this->con->escapeStr($cur_user->f('ident')).'\'') or $this->err->fatalError($this->con->error());
			}
			else {
				// If the entry is older than "o_timeout_visit", update last_visit for the user in question, then delete him/her from the online list
				if ($cur_user->f('logged') < ($now-$this->pun_config['o_timeout_visit']))
				{
					$this->con->execute('UPDATE '.DB_PREFIX.'users SET last_visit='.$cur_user->f('logged').' WHERE id='.$cur_user->f('user_id')) or $this->err->fatalError($this->con->error());
					$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE user_id='.$cur_user->f('user_id')) or $this->err->fatalError($this->con->error());
				}
				else if ($cur_user->f('idle') == '0')
				{
					$this->con->execute('UPDATE '.DB_PREFIX.'online SET idle=1 WHERE user_id='.$cur_user->f('user_id')) or $this->err->fatalError($this->con->error());
				}
			}
		}
	}

	/* ===================================================
	Cache functions
	=================================================== */

	/**
	@function _generate_config_cache
	@author Rickard Andersson from PunBB project
	
	R�g�n�re le fichier cache de configuration.
	*/
	function _generate_config_cache()
	{
		// Get the forum config from the DB
		$config = $this->con->select('SELECT * FROM '.DB_PREFIX.'config') or $this->err->fatalError($this->con->error());
		
		while ($config->fetch())
			$output[$config->f('conf_name')] = $config->f('conf_value');
		
		// Output config as PHP code
		$fh = @fopen(PUN_ROOT.'cache/cache_config.php', 'wb');
		if (!$fh)
		{
			$this->err->fatalError('Impossible d\'�crire le fichier cache de configuration dans le r�pertoire cache. Veuillez vous assurer que PHP a l\'acc�s en �criture au r�pertoire \'cache\'');
		}
		
		fwrite($fh, '<?php'."\n\n".'define(\'PUN_CONFIG_LOADED\', 1);'."\n\n".'$pun_config = '.var_export($output, true).';'."\n\n".'?>');
	
		fclose($fh);
		
		return true;
	}

	/**
	@function _generate_bans_cache
	@author Rickard Andersson from PunBB project
	
	R�g�n�re le fichier cache de bannissements
	*/
	function _generate_bans_cache()
	{
		// Get the ban list from the DB
		$cur_ban = $this->con->select('SELECT * FROM '.DB_PREFIX.'bans') or $this->err->fatalError($this->con->error());

		$output = array();
		$output = $cur_ban->getData();
		
		// Output ban list as PHP code
		$fh = @fopen(PUN_ROOT.'cache/cache_bans.php', 'wb');
		if (!$fh)
		{
			$this->err->fatalError('Impossible d\'�crire le fichier cache de bannissements dans le r�pertoire cache. Veuillez vous assurer que PHP a l\'acc�s en �criture au r�pertoire \'cache\'');
			return false;
		}
		
		fwrite($fh, '<?php'."\n\n".'define(\'PUN_BANS_LOADED\', 1);'."\n\n".'$pun_bans = '.var_export($output, true).';'."\n\n".'?>');
	
		fclose($fh);
		
		return true;
	}

//
// Equivalent to htmlspecialchars(), but allows &#[0-9]+ (for unicode)
//
function _pun_htmlspecialchars($str)
{
	$str = preg_replace('/&(?!#[0-9]+;)/s', '&amp;', $str);
	$str = str_replace(array('<', '>', '"'), array('&lt;', '&gt;', '&quot;'), $str);

	return $str;
}

	/* ===================================================
	Utilitaires
	=================================================== */

	/**
	@function logout
	
	D�connexion de l'utilisateur courant.
	@return
	*/
	function logout()
	{
		if ($this->user['is_guest'])
		{
			header('Location: '.pt_portal_url);
			exit;
		}

		// Remove user from "users online" list.
		$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE user_id='.$this->user['id']) or $this->err->fatalError($this->con->error());
	
		// Update last_visit (make sure there's something to update it with)
		if (isset($this->user['logged']))
			$this->con->execute('UPDATE '.DB_PREFIX.'users SET last_visit='.$this->user['logged'].' WHERE id='.$this->user['id']) or $this->err->fatalError($this->con->error());
		
		$this->_pun_setcookie(1, $this->_random_pass(8), time() + 31536000);
			
		return true;
	}

	/**
	@function login
	
	Identification d'un utilisateur.
	@return
	*/
	function login()
	{
		global $puntal;

		$form_username = trim($_POST['req_username']);
		$form_password = trim($_POST['req_password']);
			
		$user = $this->con->select('SELECT id, group_id, password, save_pass FROM '.DB_PREFIX.'users WHERE username=\''.$this->con->escapeStr($form_username).'\'') or $this->err->fatalError($this->con->error());
		
		$user_id = $user->f('id');
		$group_id = $user->f('group_id');
		$db_password_hash = $user->f('password');
		$save_pass = $user->f('save_pass');
	
		$authorized = false;
	
		if (!empty($db_password_hash))
		{
			$sha1_in_db = (strlen($db_password_hash) == 40) ? true : false;
			$sha1_available = (function_exists('sha1') || function_exists('mhash')) ? true : false;
	
			$form_password_hash = $this->_pun_hash($form_password);	// This could result in either an SHA-1 or an MD5 hash (depends on $sha1_available)
	
			if ($sha1_in_db && $sha1_available && $db_password_hash == $form_password_hash)
				$authorized = true;
			else if (!$sha1_in_db && $db_password_hash == md5($form_password))
			{
				$authorized = true;
	
				if ($sha1_available)	// There's an MD5 hash in the database, but SHA1 hashing is available, so we update the DB
				{
					$this->con->execute('UPDATE '.DB_PREFIX.'users SET password=\''.$form_password_hash.'\' WHERE id='.$user_id) or $this->err->fatalError($this->con->error());
				}
			}
		}
	
		if (!$authorized)
		{
			$this->err->setError(__('Wrong user/pass').' <a href="'.pt_forum_url.'login.php?action=forget">'.__('Forgotten pass').'</a>');
			return false;
		}
	
		// Update the status if this is the first time the user logged in
		if ($group_id == PUN_UNVERIFIED)
		{
			$this->con->execute('UPDATE '.DB_PREFIX.'users SET group_id='.$this->pun_config['o_default_user_group'].' WHERE id='.$user_id) or $this->err->fatalError($this->con->error());
		}
	
		// Remove this users guest entry from the online list
		$this->con->execute('DELETE FROM '.DB_PREFIX.'online WHERE ident=\''.$this->con->escapeStr($_SERVER['REMOTE_ADDR']).'\'') or $this->err->fatalError($this->con->error());
	
		$expire = ($save_pass == '1') ? time() + 31536000 : 0;
		$this->_pun_setcookie($user_id, $form_password_hash, $expire);
		
		return true;
	}

	/**
	@function redirect
	
	Affiche $message et redirige l'utilisateur vers $destination_url
	
	@param	string	redirect_url		URL de redirection
	@param	string	message				Message de redirection
	*/
	function redirect($redirect_url, $redirect_message)
	{
		if ($redirect_url == '')
			$redirect_url = pt_portal_url;
	
		// If the delay is 0 seconds, we might as well skip the redirect all together
		if ($this->pun_config['o_redirect_delay'] == '0')
		{
			header('Location: '.str_replace('&amp;', '&', $redirect_url));
			exit;
		}
		
		$GLOBALS['redirect_url'] = $redirect_url;
		$GLOBALS['redirect_message'] = $redirect_message;
		
		if (file_exists(PT_THEMES_PATH.pt_theme.'/redirect.php'))
			require PT_THEMES_PATH.pt_theme.'/redirect.php';
		else
			require PT_THEMES_PATH.PT_DEFAULT_THEME.'/redirect.php';

		# Close connexion
		$GLOBALS['con']->close();
		exit;
	}

	/**
	@function censor_words
	@author Rickard Andersson from PunBB project
	
	Replace censored words in $text
	
	@param string	text		Chaine � scanner
	@return string
	*/
	function censor_words($text)
	{
		static $search_for, $replace_with;
	
		// If not already built in a previous call, build an array of censor words and their replacement text
		if (!isset($search_for))
		{
			$words = $this->con->select('SELECT search_for, replace_with FROM '.DB_PREFIX.'censoring') or $this->err->fatalError($this->con->error());
			
			$search_for = array();
			$i = 0;
			while ($words->fetch())
			{
				$replace_with[$i] = $words->f('replace_with');
				$search_for[$i] = '/\b('.str_replace('\*', '\w*?', preg_quote($words->f('search_for'), '/')).')\b/i';
				$i++;
			}
		}
	
		if (!empty($search_for))
			$text = substr(preg_replace($search_for, $replace_with, ' '.$text.' '), 1, -1);
	
		return $text;
	}

	/**
	@function format_time
	@author Rickard Andersson from PunBB project
	
	Format a time string according to $time_format and timezones
	
	@param	string	timestamp		timestamp de la date � formater
	@param	boolean	date_only		retourner juste la date ou non (false)
	@return string
	*/
	function format_time($timestamp, $date_only = false)
	{
		global $puntal;
		
		if ($timestamp == '')
			return __('Never');
		
		// type fonction de formatage
		if (strrpos($this->pun_config['o_date_format'],'%'))
			$func = 'strftime';
		else
			$func = 'date';
		
		$diff = ($this->user['timezone'] - $this->pun_config['o_server_timezone']) * 3600;
		$timestamp += $diff;
		$now = time();
		
		$date = $func($this->pun_config['o_date_format'], $timestamp);
		$today = $func($this->pun_config['o_date_format'], $now+$diff);
		$yesterday = $func($this->pun_config['o_date_format'], $now+$diff-86400);
	
		if ($date == $today)
			$date = __('Today');
		else if ($date == $yesterday)
			$date = __('Yesterday');
	
		if (!$date_only)
			return $date.' '.$func($this->pun_config['o_time_format'], $timestamp);
		else
			return $date;
	}

	/**
	@function linebreaks
	@author Rickard Andersson from PunBB project
	
	Converti \r\n et \r en \n
	
	@param	string	string		Cha�ne � convertir
	@return string
	*/
	function linebreaks($str)
	{
		return str_replace("\r", "\n", str_replace("\r\n", "\n", $str));
	}
	
	/**
	@function getHost
	
	Return current scheme, host and port.
	*/
	function getHost()
	{
		$server_name = explode(':',$_SERVER['HTTP_HOST']);
		$server_name = $server_name[0];
		if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on')
		{
			$scheme = 'https';
			$port = ($_SERVER['SERVER_PORT'] != '443') ? ':'.$_SERVER['SERVER_PORT'] : '';
		}
		else
		{
			$scheme = 'http';
			$port = ($_SERVER['SERVER_PORT'] != '80') ? ':'.$_SERVER['SERVER_PORT'] : '';
		}
		
		return $scheme.'://'.$server_name.$port;
	}
	
	# Obtenir l'url compl�te de la page
	function getPageURL($uri=NULL)
	{
		if ($uri == NULL)
			$uri = html::escapeHTML($_SERVER['REQUEST_URI']);
		
		return puntal::getHost().$uri;
	}
	
	# Retourne une paire de l'�l�ment suivant ou pr�c�dent d'un tableau �
	# un indice donn�
	function getNextPrev($array,$key,$mode)
	{
		$keys = array_keys($array);
		$values = array_values($array);
		$p = array_flip($keys);
		$pos = !empty($p[$key]) ? $p[$key] : 0;
		
		if($mode == 'next')
			return ($pos+1 < count($array)) ? array($keys[$pos+1] => $values[$pos+1]) : NULL;
		else
			return ($pos > 0) ? array($keys[$pos-1] => $values[$pos-1]) : NULL;
	}
	
	# Couper une cha�ne aux espaces
	function cutString($str,$l)
	{
		$s = preg_split('/([\s]+)/',$str,-1,PREG_SPLIT_DELIM_CAPTURE);
		
		$res = '';
		$L = 0;
		
		if (strlen($s[0]) >= $l)
			return substr($s[0],0,$l);
		
		foreach ($s as $v)
		{
			$L = $L+strlen($v);
			
			if ($L > $l)
				break;
			else
				$res .= $v;
		}
		
		return trim($res);
	}
	
	/**
	Reconnait une cha�ne en UTF-8
	Taken from http://www.php.net/manual/fr/function.mb-detect-encoding.php#50087
	*/
	function isUTF8($string)
	{
		if (preg_match('%^(?:[\x09\x0A\x0D\x20-\x7E])*$%xs',$string))
		{
			return false;
		}
		else
		{
			// From http://w3.org/International/questions/qa-forms-utf-8.html
			return preg_match('%^(?:
				  [\x09\x0A\x0D\x20-\x7E]			# ASCII
				| [\xC2-\xDF][\x80-\xBF]				# non-overlong 2-byte
				| \xE0[\xA0-\xBF][\x80-\xBF]			# excluding overlongs
				| [\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}	# straight 3-byte
				| \xED[\x80-\x9F][\x80-\xBF]			# excluding surrogates
				| \xF0[\x90-\xBF][\x80-\xBF]{2}		# planes 1-3
				| [\xF1-\xF3][\x80-\xBF]{3}			# planes 4-15
				| \xF4[\x80-\x8F][\x80-\xBF]{2}		# plane 16
			)*$%xs', $string);
		}
	}

	/**
	@function secureString
	@author Olivier Meunier from Dotclear project
	
	Renvoie une cha�ne apr�s en avoir supprim� les tags HTML et pass� les
	caract�res sp�ciaux en entit�s.
	
	@param	string	str			Cha�ne � s�curis�
	@return	string
	*/
	function secureString($str)
	{
		$str = trim($str);
		$str = stripslashes($str);
		$str = strip_tags($str);
		$str = html::escapeHTML($str);
		return $str;
	}
	
	/**
	@function removeEntities
	@author Olivier Meunier from Dotclear project
	
	Remplace les entit�s d'une cha�ne par leurs �quivalents normaux. Les
	caract�res provenant du jeu de caract�re incorect de Windows sont �galement
	convertis.
	
	@param	string	string		Cha�ne � transformer
	@return	string
	*/
	function removeEntities($string)
	{
		# Tableau des codes de 130 � 140 et 145 � 156
		$tags = array('�' => '&sbquo;','�' => '&fnof;','�' => '&bdquo;',
		'�' => '&hellip;','�' => '&dagger;','�' => '&Dagger;','�' => '&circ;',
		'�' => '&permil;','�' => '&Scaron;','�' => '&lsaquo;','�' => '&OElig;',
		'�' => '&lsquo;','�' => '&rsquo;','�' => '&ldquo;','�' => '&rdquo;',
		'�' => '&bull;','�' => '&ndash;','�' => '&mdash;','�' => '&tilde;',
		'�' => '&trade;','�' => '&scaron;','�' => '&rsaquo;','�' => '&oelig;',
		'�' => '&Yuml;','�' => '&euro;');
		
		$vtags = array(
		'�' => '&#8218;','�' => '&#402;','�' => '&#8222;','�' => '&#8230;',
		'�' => '&#8224;','�' => '&#8225;','�' => '&#710;','�' => '&#8240;',
		'�' => '&#352;','�' => '&#8249;','�' => '&#338;','�' => '&#8216;',
		'�' => '&#8217;','�' => '&#8220;','�' => '&#8221;','�' => '&#8226;',
		'�' => '&#8211;','�' => '&#8212;','�' => '&#732;','�' => '&#8482;',
		'�' => '&#353;','�' => '&#8250;','�' => '&#339;','�' => '&#376;',
		'�' => '&#8364;');
		
		$tags = array_merge($tags,get_html_translation_table(HTML_ENTITIES));
		
		foreach($tags as $k => $v)
			$ASCIItags[$k] = '&#'.ord($k).';';
		
		$string = str_replace($tags,array_flip($tags),$string);
		$string = str_replace($ASCIItags,array_flip($ASCIItags),$string);
		$string = str_replace(array_values($vtags),array_keys($vtags),$string);
		
		return $string;
	}
	
	/**
	@function str2url
	@author Olivier Meunier from Dotclear project
	
	Transforme la cha�ne $str en une cha�ne propre � �tre un �l�ment d'une URL.
	
	@param	string	str			Cha�ne � transformer
	@return	string
	*/
	function str2url($str)
	{
		$str = $this->removeEntities($str);
		
		$str = strtr($str,
		"�������������������������������������������������������",
		"AAAAAAaaaaaaCcOOOOOOooooooEEEEeeeeIIIIiiiiUUUUuuuuYYyyNn");
		
		$str = str_replace('�','AE',$str);
		$str = str_replace('�','ae',$str);
		$str = str_replace('�','OE',$str);
		$str = str_replace('�','oe',$str);
		
		$str = preg_replace('/[^a-z0-9_\s\'\:\/\[\]-]/','',strtolower($str));
		
		$str = preg_replace('/[\s\'\:\/\[\]-]+/',' ',trim($str));
	
		$res = str_replace(' ','-',$str);
		
		return $res;
	}
	
	/**
	@function HTMLfilter
	
	Transforme la cha�ne $str en une cha�ne sans les �l�ments HTML � risque.
	
	@param	string	str			Cha�ne � transformer
	@return	string
	*/
	function HTMLfilter($str)
	{
		require_once dirname(__FILE__).'/htmlfilter.php';
		$filter = new htmlFilter(__('lang_encoding'));
		$str = trim($filter->apply(html::pt_utf8_encode($str)));
		return html::pt_utf8_decode($str);
	}
	

	/* ===================================================
	Token
	=================================================== */
	
	function setToken()
	{
		$this->ses->reg('token',$this->_pun_hash(uniqid(rand(), true)));
	}
	
	function getToken()
	{
		return $this->ses->f('token');
	}
	
	function checkToken()
	{
		if (empty($_POST['ptkn']) || $this->ses->f('token') != $_POST['ptkn'])
		{
			header('Location: '.pt_portal_url);
			exit;
		}
		
		$this->ses->destroy();
		return true;
	}
	

	/* ===================================================
	Behaviors
	=================================================== */

	/**
	@function addBehavior
	@author Olivier Meunier from Dotclear project
	
	Adds a new behavior to behaviors stack.
	
	@param string	behavior	Behavior name
	@param mixed	func		Function to call. Must be a valid callback
	*/
	function addBehavior($behavior,$func)
	{
		if (is_callable($func)) {
			$this->behaviors[$behavior][] = $func;
		}
	}

	/**
	@function hasBehavior
	@author Olivier Meunier from Dotclear project

	Tests if a particular behavior exists in behaviors stack.

	@param string	behavior	Behavior name
	@return boolean
	*/
	function hasBehavior($behavior)
	{
		return isset($this->behaviors[$behavior]);
	}

	/**
	@function getBehaviors
	@author Olivier Meunier from Dotclear project

	Get behaviors stack (or part of).

	@param string	behavior	Behavior name
	@return array
	*/
	function getBehaviors($behavior = '')
	{
		if (empty($this->behaviors)) return null;

		if ($behavior == '') {
			return $this->behaviors;
		}
		elseif (isset($this->behaviors[$behavior])) {
			return $this->behaviors[$behavior];
		}
	
		return null;
	}
	
	/**
	@function callBehavior
	@author Olivier Meunier from Dotclear project
	
	Calls every function in behaviors stack for a given behavior and returns
	concatened result of each function.
	
	@param string	behavior	Behavior name
	@return string
	*/
	function callBehavior($behavior)
	{
		if (isset($this->behaviors[$behavior]))
		{
			$args = func_get_args();
			array_shift($args);
			
			$res = '';
			
			foreach ($this->behaviors[$behavior] as $f) {
				$res .= call_user_func_array($f,$args);
			}
			
			return $res;
		}
	}
	
}

?>