<?php
# ***** BEGIN LICENSE BLOCK *****
# This file is part of Clearbricks.
# Copyright (c) 2004 Olivier Meunier and contributors. All rights
# reserved.
#
# Clearbricks is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# Clearbricks is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Clearbricks; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# ***** END LICENSE BLOCK *****

class html
{
	/**
	@function escapeHTML
	
	Replaces HTML special characters by entities.
	
	@param	str	string	String to escape
	@return	string
	*/
	function escapeHTML($str)
	{
		return htmlspecialchars($str,ENT_COMPAT,__('lang_encoding'));
	}
	
	/**
	@function decodeEntities
	
	Returns a string with all entities decoded.
	
	@param	str	string	String to protect
	@param	keep_special string		Keep special characters: &gt; &lt; &amp;
	@return	string
	*/
	function decodeEntities($str,$keep_special=false)
	{
		if ($keep_special) {
			$str = str_replace(
				array('&amp;','&gt;','&lt;'),
				array('&amp;amp;','&amp;gt;','&amp;lt;'),
				$str);
		}
		
		# Some extra replacements
		$extra = array(
			'&apos;' => "'"
		);
		
		$str = str_replace(array_keys($extra),array_values($extra),$str);
		
		return html_entity_decode($str,ENT_QUOTES,__('lang_encoding'));
	}
	
	/**
	@function clean
	
	Removes every tags, comments, cdata from string
	
	@param	str	string	String to clean
	@return	string
	*/
	function clean($str)
	{
		$str = strip_tags($str);
		return $str;
	}
	
	/**
	@function escapeJS
	
	Returns a protected JavaScript string
	
	@param	string	str			String to protect
	@return	string
	*/
	function escapeJS($str)
	{
		$str = htmlspecialchars($str,ENT_NOQUOTES,__('lang_encoding'));
		$str = str_replace("'","\'",$str);
		$str = str_replace('"','\"',$str);
		return $str;
	}
	
	/**
	@function escapeURL
	
	Returns an escaped URL string for HTML content
	
	@param	str	string	String to escape
	@return	string
	*/
	function escapeURL($str)
	{
		return str_replace('&','&amp;',$str);
	}
	
	/**
	@function sanitizeURL
	
	Encode every parts between / in url
	
	@param	str	string	String to satinyze
	@return	string
	*/
	function sanitizeURL($str)
	{
		return str_replace('%2F','/',rawurlencode($str));
	}
	
	/**
	@function stripHostURL
	
	Removes host part in URL
	
	@param	url	string	URL to transform
	@return	string
	*/
	function stripHostURL($url)
	{
		return preg_replace('|^[a-z]{3,}://.*?(/.*$)|','$1',$url);
	}
	
	
	function pt_utf8_encode($str)
	{
		if (strtoupper(__('lang_encoding')) == 'UTF-8')
			return $str;
		else
			return utf8_encode($str);
	}
	
	function pt_utf8_decode($str)
	{
		if (strtoupper(__('lang_encoding')) == 'UTF-8')
			return $str;
		else
			return utf8_decode($str);
	}

}
?>