<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

/*

Class statique, sorte d'API de PunBB

*/
class punbb
{
	
	#------------------------------------------------
	# Get datas methods
	#------------------------------------------------
	
	/**
	@function getForums
	
	Retourne un recordset contenant tous les forums et
	toutes les catgories.
	
	@return recordset
	*/
	function getForums()
	{
		global $puntal;
		
		$strReq = 
		'SELECT c.id AS cid, c.cat_name, f.id AS fid, f.forum_name '.
		'FROM '.DB_PREFIX.'categories AS c '.
		'INNER JOIN '.DB_PREFIX.'forums AS f ON c.id=f.cat_id '.
		'ORDER BY c.disp_position, c.id, f.disp_position';
				
		$rs = $puntal->con->select($strReq) or $puntal->err->fatalError($puntal->con->error());
		
		return $rs;		
	}


	/**
	@function getGroups
	
	Retourne un recordset contenant tous les groupes.
	
	@return recordset
	*/
	function getGroups()
	{
		global $puntal;
		
		$strReq = 
		'SELECT g_id, g_title '.
		'FROM '.DB_PREFIX.'groups '.
		'ORDER BY g_id';
				
		$rs = $puntal->con->select($strReq) or $puntal->err->fatalError($puntal->con->error());
		
		return $rs;		
	}
	
	
	/**
	@function getTopicPosts
	
	Retourne un recordset contenant tous les posts d'un topic donn.
	
	@param	integer	tid			Identifiant du topic
	@param	string	limit		Restrendre  un certain nombre de posts ('')
	@param	boolean pid			Post  ne pas retrouner (util pour ne pas prendre le premier post d'un topic) (0)
	@return recordset
	*/
	function getTopicPosts($tid,$limit='',$pid=0)
	{
		global $puntal;
		
		$strReq = 
		'SELECT u.email, u.title, u.url, u.location, u.use_avatar, u.signature, '.
		'u.email_setting, u.num_posts, u.registered, u.admin_note, '.
		'p.id, p.poster AS username, p.poster_id, p.poster_ip, p.poster_email, '.
		'p.message, p.hide_smilies, p.posted, p.edited, p.edited_by, '.
		'g.g_id, g.g_user_title, '.
		'o.user_id AS is_online '.
		'FROM '.DB_PREFIX.'posts AS p '.
		'	INNER JOIN '.DB_PREFIX.'users AS u ON u.id=p.poster_id '.
		'	INNER JOIN '.DB_PREFIX.'groups AS g ON g.g_id=u.group_id '.
		'	LEFT JOIN '.DB_PREFIX.'online AS o ON (o.user_id=u.id AND o.user_id!=1 AND o.idle=0) '.
		'WHERE p.topic_id='.$tid.' '.
		($pid > 0 ? ' AND p.id != '.$pid.' ' : '').
		'ORDER BY p.id';
		
		if ($limit != '')
			$strReq .= ' LIMIT '.$limit;
				
		$rs = $puntal->con->select($strReq) or $puntal->err->fatalError($puntal->con->error());
		
		return $rs;		
	}

	
	#------------------------------------------------
	# Post methods
	#------------------------------------------------

	/**
	@function newTopic
	
	Lance un sujet dans un forum et retourne
	les ID du sujet et du message crs :
	
	array('tid' => $new_tid, 'pid' => $new_pid);
	
	@param	integer	forum_id		ID du forum dans lequel il faut poster
	@param	string	subject			le sujet du topic
	@param	string	message			le message du topic
	@param	boolean	increment		incremente ou non le compteur de post de l'utilisateur (true)
	@param	boolean	hide_smilies	transforme les smiley ou non (false)
	@param	boolean	subscribe		abonnement  la discussion ou non (false)
	@param	string	username		le nom d'utilisateur ('')
	@param	string	email			l'adresse email de l'utilisateur ('')
	@param	boolean	bypassperm		passe outre les permissions (false)
	@return array('tid' => $new_tid, 'pid' => $new_pid);
	*/
	function newTopic($forum_id,$subject,$message,$increment=true,$hide_smilies=false,$subscribe=false,$username='',$email='',$bypassperm=false)
	{
		global $puntal;
		
		// Transforme hide_smiley and subscirbe bool in integer for sql query
		$hide_smilies = ($hide_smilies == true || $hide_smilies == 1) ? 1 : 0;
		$subscribe = ($subscribe == true || $subscribe == 1) ? 1 : 0;
		
		// Fetch some infos
		$cur_posting = $puntal->con->select('SELECT f.id, f.forum_name, f.moderators, f.redirect_url, fp.post_replies, fp.post_topics FROM '.DB_PREFIX.'forums AS f LEFT JOIN '.DB_PREFIX.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$puntal->user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND f.id='.$forum_id) or $puntal->err->fatalError($puntal->con->error());
			
		if ($cur_posting->isEmpty())
		{
			$puntal->err->setError(__('Bad request. The link you followed is incorrect or outdated.'));
			return false;
		}
		
		// Is someone trying to post into a redirect forum?
		if ($cur_posting->f('redirect_url') != '')
		{
			$puntal->err->setError(__('Bad request. The link you followed is incorrect or outdated.'));
			return false;
		}
		
		// Sort out who the moderators are and if we are currently a moderator (or an admin)
		$mods_array = ($cur_posting->f('moderators') != '') ? unserialize($cur_posting->f('moderators')) : array();
		$is_admmod = ($puntal->user['g_id'] == PUN_ADMIN || ($puntal->user['g_id'] == PUN_MOD && array_key_exists($puntal->user['username'], $mods_array))) ? true : false;
		
		// Do we have permission to post?		
		$post_topic = (string)$cur_posting->f('post_topics');
		if (!$bypassperm && !$is_admmod && ((($post_topic === '' && $puntal->user['g_post_topics'] == '0') || $post_topic === '0')))
		{
			$puntal->err->setError(__('You do not have permission to access this page.'));
			return false;
		}
		
		$lang_post = array();
		// Load the post.php language file
		require PUN_ROOT.'lang/'.$puntal->user['language'].'/post.php';
		
		// Flood protection
		if (!$puntal->user['is_guest'] && $puntal->user['last_post'] != '' && (time() - $puntal->user['last_post']) < $puntal->user['g_post_flood'])
		{
			$puntal->err->setError($lang_post['Flood start'].' '.$puntal->user['g_post_flood'].' '.$lang_post['flood end']);
			return false;
		}
		
		// Check user infos
		if (!punbb::prePostUser($username,$email))
			return false;
		
		// Check title
		if (!punbb::prePostTitle($subject,$lang_post))
			return false;
		
		// Check and clean up message
		if (!punbb::prePostMessage($message,$lang_post))
			return false;
		
		// Did everything go according to plan?
		if ($puntal->err->hasError())
			return false;
		else 
		{
			$now = time();
			
			// Create the topic
			$puntal->con->execute('INSERT INTO '.DB_PREFIX.'topics (poster, subject, posted, last_post, last_poster, forum_id) VALUES(\''.$puntal->con->escapeStr($username).'\', \''.$puntal->con->escapeStr($subject).'\', '.$now.', '.$now.', \''.$puntal->con->escapeStr($username).'\', '.$forum_id.')') or $puntal->err->fatalError($puntal->con->error());
			
			$new_tid = $puntal->con->getLastID();

			if (!$puntal->user['is_guest'])
			{
				// To subscribe or not to subscribe, that ...
				if ($puntal->pun_config['o_subscriptions'] == '1' && $subscribe == 1)
				{
					$puntal->con->execute('INSERT INTO '.DB_PREFIX.'subscriptions (user_id, topic_id) VALUES('.$puntal->user['id'].' ,'.$new_tid.')') or $puntal->err->fatalError($puntal->con->error());
				}
				
				// Create the post ("topic post")
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'posts (poster, poster_id, poster_ip, message, hide_smilies, posted, topic_id) VALUES(\''.$puntal->con->escapeStr($username).'\', '.$puntal->user['id'].', \''.$_SERVER['REMOTE_ADDR'].'\', \''.$puntal->con->escapeStr($message).'\', \''.$hide_smilies.'\', '.$now.', '.$new_tid.')') or $puntal->err->fatalError($puntal->con->error());
				
				$new_pid = $puntal->con->getLastID();
			}
			else {
				// Create the post ("topic post")
				$email_sql = ($puntal->pun_config['p_force_guest_email'] == '1' || $email != '') ? '\''.$email.'\'' : 'NULL';
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'posts (poster, poster_ip, poster_email, message, hide_smilies, posted, topic_id) VALUES(\''.$puntal->con->escapeStr($username).'\', \''.$_SERVER['REMOTE_ADDR'].'\', '.$email_sql.', \''.$puntal->con->escapeStr($message).'\', \''.$hide_smilies.'\', '.$now.', '.$new_tid.')') or $puntal->err->fatalError($puntal->con->error());
				
				$new_pid = $puntal->con->getLastID();
			}
			
			// Update the topic with last_post_id
			$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET last_post_id='.$new_pid.' WHERE id='.$new_tid) or $puntal->err->fatalError($puntal->con->error());
							
			punbb::updateSearchIndex('post', $new_pid, $message, $subject);
	
			punbb::updateForum($forum_id);

			// If the posting user is logged in, increment his/her post count
			if (!$puntal->user['is_guest'] && $increment)
			{
				$puntal->con->execute('UPDATE LOW_PRIORITY '.DB_PREFIX.'users SET num_posts=num_posts+1, last_post='.$now.' WHERE id='.$puntal->user['id']) or $puntal->err->fatalError($puntal->con->error());
			}
			
			return array('tid' => $new_tid, 'pid' => $new_pid);
		}
	}
	
	
	/**
	@function newPost
	
	Ajoute un message  une discussions tid et retourne l'ID du nouveau message
	
	@param	integer	tid				ID du topic dans lequel il faut poster
	@param	string	message			le message du post
	@param	boolean	increment		incremente ou non le compteur de post de l'utilisateur (true)
	@param	boolean	hide_smilies	transforme les smiley ou non (false)
	@param	boolean	subscribe		abonnement  la discussion ou non (false)
	@param	string	username		le nom d'utilisateur ('')
	@param	string	email			l'adresse email de l'utilisateur ('')
	@param	boolean	bypassperm		passe outre les permissions (false)
	@return integer
	*/
	function newPost($tid,$message,$increment=true,$hide_smilies=false,$subscribe=false,$username='',$email='',$bypassperm=false)
	{
		global $puntal;
		
		// Transforme hide_smiley and subscirbe bool in integer for sql query
		$hide_smilies = ($hide_smilies == true || $hide_smilies == 1) ? 1 : 0;
		$subscribe = ($subscribe == true || $subscribe == 1) ? 1 : 0;
		
		// Fetch some infos
		$cur_posting = $puntal->con->select($strReq = 'SELECT f.id, f.forum_name, f.moderators, f.redirect_url, fp.post_replies, fp.post_topics, t.subject, t.closed FROM '.DB_PREFIX.'topics AS t INNER JOIN '.DB_PREFIX.'forums AS f ON f.id=t.forum_id LEFT JOIN '.DB_PREFIX.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$puntal->user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND t.id='.$tid) or $puntal->err->fatalError($puntal->con->error());
					
		if ($cur_posting->isEmpty())
		{
			$puntal->err->setError(__('Bad request. The link you followed is incorrect or outdated.'));
			return false;
		}
		
		// Is someone trying to post into a redirect forum?
		if ($cur_posting->f('redirect_url') != '')
		{
			$puntal->err->setError(__('Bad request. The link you followed is incorrect or outdated.'));
			return false;
		}
		
		// Sort out who the moderators are and if we are currently a moderator (or an admin)
		$mods_array = ($cur_posting->f('moderators') != '') ? unserialize($cur_posting->f('moderators')) : array();
		$is_admmod = ($puntal->user['g_id'] == PUN_ADMIN || ($puntal->user['g_id'] == PUN_MOD && array_key_exists($puntal->user['username'], $mods_array))) ? true : false;
		
		// Do we have permission to post?		
		if (!$bypassperm && !$is_admmod && ((($cur_posting->f('post_replies') == '' && $puntal->user->f('g_post_replies') == '0') || $cur_posting->f('post_replies') == '0') || (isset($cur_posting['closed']) && $cur_posting['closed'] == '1')))
		{
			$puntal->err->setError(__('You do not have permission to access this page.'));
			return false;
		}
		
		$lang_post = array();
		// Load the post.php language file
		require PUN_ROOT.'lang/'.$puntal->user['language'].'/post.php';
		
		// Flood protection
		if (!$puntal->user['is_guest'] && $puntal->user['last_post'] != '' && (time() - $puntal->user['last_post']) < $puntal->user['g_post_flood'])
		{
			$puntal->err->setError($lang_post['Flood start'].' '.$puntal->user['g_post_flood'].' '.$lang_post['flood end']);
			return false;
		}
		
		// Check user infos
		if (!punbb::prePostUser($username,$email))
			return false;
				
		// Clean up message
		if (!punbb::prePostMessage($message,$lang_post))
			return false;
		
		// Did everything go according to plan?
		if ($puntal->err->hasError())
			return false;
		else 
		{
			$now = time();
			if (!$puntal->user['is_guest'])
			{
				// Insert the new post
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'posts (poster, poster_id, poster_ip, message, hide_smilies, posted, topic_id) VALUES(\''.$puntal->con->escapeStr($username).'\', '.$puntal->user['id'].', \''.$_SERVER['REMOTE_ADDR'].'\', \''.$puntal->con->escapeStr($message).'\', \''.$hide_smilies.'\', '.$now.', '.$tid.')') or $puntal->err->fatalError($puntal->con->error());
				
				$new_pid = $puntal->con->getLastID();

				// To subscribe or not to subscribe, that ...
				if ($puntal->pun_config['o_subscriptions'] == '1' && $subscribe)
				{
					$result = $puntal->con->select('SELECT 1 FROM '.DB_PREFIX.'subscriptions WHERE user_id='.$puntal->user['id'].' AND topic_id='.$tid) or $puntal->err->fatalError($puntal->con->error());
					
					if ($result->isEmpty())
						$puntal->con->execute('INSERT INTO '.DB_PREFIX.'subscriptions (user_id, topic_id) VALUES('.$puntal->user['id'].' ,'.$tid.')') or $puntal->err->fatalError($puntal->con->error());
				}
			}
			else
			{
				// It's a guest. Insert the new post
				$email_sql = ($puntal->pun_config['p_force_guest_email'] == '1' || $email != '') ? '\''.$email.'\'' : 'NULL';
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'posts (poster, poster_ip, poster_email, message, hide_smilies, posted, topic_id) VALUES(\''.$puntal->con->escapeStr($username).'\', \''.$_SERVER['REMOTE_ADDR'].'\', '.$email_sql.', \''.$puntal->con->escapeStr($message).'\', \''.$hide_smilies.'\', '.$now.', '.$tid.')') or $puntal->err->fatalError($puntal->con->error());
				
				$new_pid = $puntal->con->getLastID();
			}

			// Count number of replies in the topic
			$result = $puntal->con->select('SELECT COUNT(id) AS num_replies FROM '.DB_PREFIX.'posts WHERE topic_id='.$tid) or $puntal->err->fatalError($puntal->con->error());
			$num_replies = $result->f('num_replies') - 1;

			// Update topic
			$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET num_replies='.$num_replies.', last_post='.$now.', last_post_id='.$new_pid.', last_poster=\''.$puntal->con->escapeStr($username).'\' WHERE id='.$tid) or $puntal->err->fatalError($puntal->con->error());

			punbb::updateSearchIndex('post', $new_pid, $message);

			punbb::updateForum($cur_posting->f('id'));

			// Should we send out notifications?
			if ($puntal->pun_config['o_subscriptions'] == '1')
			{
				// Get the post time for the previous post in this topic
				$result = $puntal->con->select('SELECT posted FROM '.DB_PREFIX.'posts WHERE topic_id='.$tid.' ORDER BY id DESC LIMIT 1, 1') or $puntal->err->fatalError($puntal->con->error());
				$previous_post_time = $result->f('posted');

				// Get any subscribed users that should be notified (banned users are excluded)
				$subscriber = $puntal->con->select('SELECT u.id, u.email, u.notify_with_post, u.language FROM '.DB_PREFIX.'users AS u INNER JOIN '.DB_PREFIX.'subscriptions AS s ON u.id=s.user_id LEFT JOIN '.DB_PREFIX.'forum_perms AS fp ON (fp.forum_id='.$cur_posting->f('id').' AND fp.group_id=u.group_id) LEFT JOIN '.DB_PREFIX.'online AS o ON u.id=o.user_id LEFT JOIN '.DB_PREFIX.'bans AS b ON u.username=b.username WHERE b.username IS NULL AND COALESCE(o.logged, u.last_visit)>'.$previous_post_time.' AND (fp.read_forum IS NULL OR fp.read_forum=1) AND s.topic_id='.$tid.' AND u.id!='.intval($puntal->user['id'])) or $puntal->err->fatalError($puntal->con->error());
				
				if (!$subscriber->isEmpty())
				{
					require PT_INC_PATH.'libs/email.php';
					
					$notification_emails = array();

					// Loop through subscribed users and send e-mails
					while ($subscriber->fetch())
					{
						// Is the subscription e-mail for $subscriber->f('language') cached or not?
						if (!isset($notification_emails[$subscriber->f('language')]))
						{
							if (file_exists(PUN_ROOT.'lang/'.$subscriber->f('language').'/mail_templates/new_reply.tpl'))
							{
								// Load the "new reply" template
								$mail_tpl = trim(file_get_contents(PUN_ROOT.'lang/'.$subscriber->f('language').'/mail_templates/new_reply.tpl'));

								// Load the "new reply full" template (with post included)
								$mail_tpl_full = trim(file_get_contents(PUN_ROOT.'lang/'.$subscriber->f('language').'/mail_templates/new_reply_full.tpl'));

								// The first row contains the subject (it also starts with "Subject:")
								$first_crlf = strpos($mail_tpl, "\n");
								$mail_subject = trim(substr($mail_tpl, 8, $first_crlf-8));
								$mail_message = trim(substr($mail_tpl, $first_crlf));

								$first_crlf = strpos($mail_tpl_full, "\n");
								$mail_subject_full = trim(substr($mail_tpl_full, 8, $first_crlf-8));
								$mail_message_full = trim(substr($mail_tpl_full, $first_crlf));

								$mail_subject = str_replace('<topic_subject>', '\''.$cur_posting->f('subject').'\'', $mail_subject);
								$mail_message = str_replace('<topic_subject>', '\''.$cur_posting->f('subject').'\'', $mail_message);
								$mail_message = str_replace('<replier>', $username, $mail_message);
								$mail_message = str_replace('<post_url>', $puntal->pun_config['o_base_url'].'/viewtopic.php?pid='.$new_pid.'#p'.$new_pid, $mail_message);
								$mail_message = str_replace('<unsubscribe_url>', $puntal->pun_config['o_base_url'].'/misc.php?unsubscribe='.$tid, $mail_message);
								$mail_message = str_replace('<board_mailer>', $puntal->pun_config['o_board_title'].' '.__('Mailer'), $mail_message);

								$mail_subject_full = str_replace('<topic_subject>', '\''.$cur_posting->f('subject').'\'', $mail_subject_full);
								$mail_message_full = str_replace('<topic_subject>', '\''.$cur_posting->f('subject').'\'', $mail_message_full);
								$mail_message_full = str_replace('<replier>', $username, $mail_message_full);
								$mail_message_full = str_replace('<message>', $message, $mail_message_full);
								$mail_message_full = str_replace('<post_url>', $puntal->pun_config['o_base_url'].'/viewtopic.php?pid='.$new_pid.'#p'.$new_pid, $mail_message_full);
								$mail_message_full = str_replace('<unsubscribe_url>', $puntal->pun_config['o_base_url'].'/misc.php?unsubscribe='.$tid, $mail_message_full);
								$mail_message_full = str_replace('<board_mailer>', $puntal->pun_config['o_board_title'].' '.__('Mailer'), $mail_message_full);

								$notification_emails[$subscriber->f('language')][0] = $mail_subject;
								$notification_emails[$subscriber->f('language')][1] = $mail_message;
								$notification_emails[$subscriber->f('language')][2] = $mail_subject_full;
								$notification_emails[$subscriber->f('language')][3] = $mail_message_full;

								$mail_subject = $mail_message = $mail_subject_full = $mail_message_full = null;
							}
						}

						// We have to double check here because the templates could be missing
						if (isset($notification_emails[$subscriber->f('language')]))
						{
							if ($subscriber->f('notify_with_post') == '0')
								pun_mail($subscriber->f('email'), $notification_emails[$subscriber->f('language')][0], $notification_emails[$subscriber->f('language')][1]);
							else
								pun_mail($subscriber->f('email'), $notification_emails[$subscriber->f('language')][2], $notification_emails[$subscriber->f('language')][3]);
						}
					}
				}
			}
		}
		
		return $new_pid;
	}
	
	
	/*
	@function editPost
	
	Modification d'un message
	
	@param integer id				ID du post  modifier
	@param string message			Le nouveau message
	@param string subject			Sujet si c'est un post-topic ('')
	@param boolean hide_smilies		transforme les smiley ou non (false)
	@param boolean silent			Modification silencieuse (false)
	@param boolean bypassperm		passe outre les permissions (false)
	*/
	function editPost($id,$message,$subject='',$hide_smilies=false,$silent=false,$bypassperm=false)
	{
		global $puntal;
		
		// Transforme hide_smiley bool in integer for sql query
		$hide_smilies = ($hide_smilies == true || $hide_smilies == 1) ? 1 : 0;

		// Fetch some info about the post, the topic and the forum
		$cur_post = $puntal->con->select('SELECT f.id AS fid, f.forum_name, f.moderators, f.redirect_url, fp.post_replies, fp.post_topics, t.id AS tid, t.subject, t.posted, t.closed, p.poster, p.poster_id, p.message, p.hide_smilies FROM '.DB_PREFIX.'posts AS p INNER JOIN '.DB_PREFIX.'topics AS t ON t.id=p.topic_id INNER JOIN '.DB_PREFIX.'forums AS f ON f.id=t.forum_id LEFT JOIN '.DB_PREFIX.'forum_perms AS fp ON (fp.forum_id=f.id AND fp.group_id='.$puntal->user['g_id'].') WHERE (fp.read_forum IS NULL OR fp.read_forum=1) AND p.id='.$id) or $puntal->err->fatalError($puntal->con->error());
			
		if ($cur_post->isEmpty())
		{
			$puntal->err->setError(__('Bad request. The link you followed is incorrect or outdated.'));
			return false;
		}
		
		// Sort out who the moderators are and if we are currently a moderator (or an admin)
		$mods_array = ($cur_post->f('moderators') != '') ? unserialize($cur_post->f('moderators')) : array();
		$is_admmod = ($puntal->user['g_id'] == PUN_ADMIN || ($puntal->user['g_id'] == PUN_MOD && array_key_exists($puntal->user['username'], $mods_array))) ? true : false;
		
		// Determine whether this post is the "topic post" or not
		$rs = $puntal->con->select('SELECT id FROM '.DB_PREFIX.'posts WHERE topic_id='.$cur_post->f('tid').' ORDER BY posted LIMIT 1') or $puntal->err->fatalError($puntal->con->error());
		$topic_post_id = $rs->f('id');
		
		$can_edit_subject = ($id == $topic_post_id && (($puntal->user['g_edit_subjects_interval'] == '0' || (time() - $cur_post->f('posted')) < $puntal->user['g_edit_subjects_interval']) || $is_admmod)) ? true : false;
		
		// Do we have permission to edit this post?
		if (($puntal->user['g_edit_posts'] == '0' ||
			$cur_post->f('poster_id') != $puntal->user['id'] ||
			$cur_post->f('closed') == '1') &&
			!$is_admmod && !$bypassperm)
		{
			$puntal->err->setError(__('You do not have permission to access this page.'));
			return false;
		}
		
		$lang_post = array();
		// Load the post.php language file
		require PUN_ROOT.'lang/'.$puntal->user['language'].'/post.php';
		
		// If it is a topic it must contain a subject and we have to check it
		if ($can_edit_subject)
		{
			if (!punbb::prePostTitle($subject,$lang_post))
				return false;
		}
		
		// Clean up message
		if (!punbb::prePostMessage($message,$lang_post))
			return false;
		
		// Did everything go according to plan?
		if ($puntal->err->hasError())
			return false;
		else 
		{
			$edited_sql = (!$silent || !$is_admmod) ? $edited_sql = ', edited='.time().', edited_by=\''.$puntal->con->escapeStr($puntal->user['username']).'\'' : '';
	
			if ($can_edit_subject)
			{
				// Update the topic and any redirect topics
				$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET subject=\''.$puntal->con->escapeStr($subject).'\' WHERE id='.$cur_post->f('tid').' OR moved_to='.$cur_post->f('tid')) or $puntal->err->fatalError($puntal->con->error());
	
				// We changed the subject, so we need to take that into account when we update the search words
				punbb::updateSearchIndex('edit', $id, $message, $subject);
			}
			else
				punbb::updateSearchIndex('edit', $id, $message);
	
			// Update the post
			$puntal->con->execute('UPDATE '.DB_PREFIX.'posts SET message=\''.$puntal->con->escapeStr($message).'\', hide_smilies=\''.$hide_smilies.'\''.$edited_sql.' WHERE id='.$id) or $puntal->err->fatalError($puntal->con->error());
		}
	
		return true;
	}
	
		
	/*
	@function prePostUser
	
	Test le nom et l'email utilisateur avant de poster
	
	@param string username (ref)
	@param string email (ref)
	*/
	function prePostUser(&$username,&$email)
	{
		global $puntal;
		
		// If the user is logged in we get the username and e-mail from $puntal->user
		if (!$puntal->user['is_guest'])
		{
			$username = $puntal->user['username'];
			$email = $puntal->user['email'];
		}
		// Otherwise it should be in $_POST
		else
		{
			$username = trim($username);
			$email = strtolower(trim($email));
	
			$lang_prof_reg = array();
			$lang_register= array();
			// Load the register.php/profile.php language files
			require PUN_ROOT.'lang/'.$puntal->user['language'].'/prof_reg.php';
			require PUN_ROOT.'lang/'.$puntal->user['language'].'/register.php';
	
			// It's a guest, so we have to validate the username
			if (strlen($username) < 2)
			{
				$puntal->err->setError($lang_prof_reg['Username too short']);
				return false;
			}
			else if (!strcasecmp($username, 'invit') || !strcasecmp($username, 'guest') || !strcasecmp($username, $puntal->lang['Guest']))
			{
				$puntal->err->setError($lang_prof_reg['Username guest']);
				return false;
			}
			else if (preg_match('/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/', $username))
			{
				$puntal->err->setError($lang_prof_reg['Username IP']);
				return false;
			}
	
			if ((strpos($username, '[') !== false || strpos($username, ']') !== false) && strpos($username, '\'') !== false && strpos($username, '"') !== false)
			{
				$puntal->err->setError($lang_prof_reg['Username reserved chars']);
				return false;
			}

			if (preg_match('#\[b\]|\[/b\]|\[u\]|\[/u\]|\[i\]|\[/i\]|\[color|\[/color\]|\[quote\]|\[quote=|\[/quote\]|\[code\]|\[/code\]|\[img\]|\[/img\]|\[url|\[/url\]|\[email|\[/email\]#i', $username))
			{
				$puntal->err->setError($lang_prof_reg['Username BBCode']);
				return false;
			}
	
			// Check username for any censored words
			$temp = $puntal->censor_words($username);
			if ($temp != $username)
			{
				$puntal->err->setError($lang_register['Username censor']);
				return false;
			}
	
			// Check that the username (or a too similar username) is not already registered
			$busy = $puntal->con->select('SELECT username FROM '.DB_PREFIX.'users WHERE username=\''.$puntal->con->escapeStr($username).'\' OR username=\''.$puntal->con->escapeStr(preg_replace('/[^\w]/', '', $username)).'\'') or $puntal->err->fatalError($puntal->con->error());
			
			if (!$busy->isEmpty())
			{
				$puntal->err->setError($lang_register['Username dupe 1'].' '.html::escapeHTML($busy->f('username')).'. '.$lang_register['Username dupe 2']);
				return false;
			}
	
			if ($puntal->pun_config['p_force_guest_email'] == '1' || $email != '')
			{
				require dirname(__FILE__).'/email.php';
				if (!is_valid_email($email))
				{
					$puntal->err->setError($puntal->lang['Invalid e-mail']);
					return false;
				}
			}
		}
		return true;
	}
	
	
	/**
	@function prePostTitle
	 
	Prpare le title d'un topic avant de poster
	 
	@param string subject (ref)		Le titre
	@param array lang_post (ref)	Le tableau de langue de post
	@return boolean
	*/
	function prePostTitle(&$subject,&$lang_post)
	{
		global $puntal;
		
		$subject = trim($subject);

		if ($subject == '')
		{
			$puntal->err->setError($lang_post['No subject']);
			return false;
		}
		else if (strlen($subject) > 70)
		{
			$puntal->err->setError($lang_post['Too long subject']);
			return false;
		}
		else if ($puntal->pun_config['p_subject_all_caps'] == '0' && strtoupper($subject) == $subject && $puntal->user['g_id'] > PUN_MOD)
		{
			$subject = ucwords(strtolower($subject));
		}
		
		return true;
	}
	
	
	/**
	@function prePostMessage
	 
	Prpare le message d'un nouveau message avant de poster
	 
	@param	string	message	(ref)	Message
	@param array lang_post (ref)	Le tableau de langue de post
	@return boolean
	*/
	function prePostMessage(&$message,&$lang_post)
	{
		global $puntal;
		
		$message = $puntal->linebreaks(trim($message));
	
		if ($message == '')
		{
			$puntal->err->setError($lang_post['No message']);
			return false;
		}
		else if (strlen($message) > 65535)
		{
			$puntal->err->setError($lang_post['Too long message']);
			return false;
		}
		else if ($puntal->pun_config['p_message_all_caps'] == '0' && strtoupper($message) == $message && $puntal->user['g_id'] > PUN_MOD)
			$message = ucwords(strtolower($message));
	
		// Validate BBCode syntax
		if ($puntal->pun_config['p_message_bbcode'] == '1' && strpos($message, '[') !== false && strpos($message, ']') !== false)
		{
			require_once dirname(__FILE__).'/punbb_parser.php';
			$message = preparse_bbcode($message);
		}
		
		return true;
	}
	
	
	/**
	@function delTopic
	 
	Supprime une discussion et tous ses messages
	 
	@param	integer	topic_id		ID de la discussion a supprimer
	@return boolean
	*/
	function delTopic($topic_id)
	{
		global $puntal;
		
		// Delete the topic and any redirect topics
		$puntal->con->execute('DELETE FROM '.DB_PREFIX.'topics WHERE id='.$topic_id.' OR moved_to='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
	
		// Create a list of the post ID's in this topic
		$post_ids = '';
		$row = $puntal->con->select('SELECT id FROM '.DB_PREFIX.'posts WHERE topic_id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
		
		while ($row->fetch())
			$post_ids .= ($post_ids != '') ? ','.$row->f('id') : $row->f('id');
	
		// Make sure we have a list of post ID's
		if ($post_ids != '')
		{
			punbb::stripSearchIndex($post_ids);
	
			// Delete posts in topic
			$puntal->con->execute('DELETE FROM '.DB_PREFIX.'posts WHERE topic_id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
		}
	
		// Delete any subscriptions for this topic
		$puntal->con->execute('DELETE FROM '.DB_PREFIX.'subscriptions WHERE topic_id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
		
		return true;
	}
		
	
	/**
	@function delPost
	 
	Supprime un message donn
	 
	@param	integer	post_id			ID du message  supprimer
	@param	integer	topic_id		ID de la discussion du message
	@return boolean
	*/
	function delPost($post_id, $topic_id)
	{
		global $puntal;
	
		$rs = $puntal->con->select('SELECT id, poster, posted FROM '.DB_PREFIX.'posts WHERE topic_id='.$topic_id.' ORDER BY id DESC LIMIT 2') or $puntal->err->fatalError($puntal->con->error());
		
		$rs->moveStart();
		$last_id = $rs->f('id');
		$rs->moveNext();
		$second_last_id = $rs->f('id');
		$second_poster = $rs->f('poster');
		$second_posted = $rs->f('posted');
	
		// Delete the post
		$puntal->con->execute('DELETE FROM '.DB_PREFIX.'posts WHERE id='.$post_id) or $puntal->err->fatalError($puntal->con->error());
		
		punbb::stripSearchIndex($post_id);
	
		// Count number of replies in the topic
		$rs = $puntal->con->select('SELECT COUNT(id) AS num_replies FROM '.DB_PREFIX.'posts WHERE topic_id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
		$num_replies = $rs->f('num_replies') - 1;
	
		// If the message we deleted is the most recent in the topic (at the end of the topic)
		if ($last_id == $post_id)
		{
			// If there is a $second_last_id there is more than 1 reply to the topic
			if (!empty($second_last_id))
			{
				$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET last_post='.$second_posted.', last_post_id='.$second_last_id.', last_poster=\''.$puntal->con->escapeStr($second_poster).'\', num_replies='.$num_replies.' WHERE id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
			}
			else {
				// We deleted the only reply, so now last_post/last_post_id/last_poster is posted/id/poster from the topic itself
				$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET last_post=posted, last_post_id=id, last_poster=poster, num_replies='.$num_replies.' WHERE id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
			}
		}
		else {
			// Otherwise we just decrement the reply counter
			$puntal->con->execute('UPDATE '.DB_PREFIX.'topics SET num_replies='.$num_replies.' WHERE id='.$topic_id) or $puntal->err->fatalError($puntal->con->error());
		}
		
		return true;
	}
	
	function checkPostTopic($id, $tid)
	{
		global $puntal;
	
		$strReq = 
		'SELECT t.id AS tid, p.message '.
		'FROM '.DB_PREFIX.'posts AS p '.
		'	INNER JOIN '.DB_PREFIX.'topics AS t ON t.id=p.topic_id '.
		'WHERE p.id='.$id;
		
		$cur_post = $puntal->con->select($strReq) or $puntal->err->fatalError($puntal->con->error());
			
		if ($cur_post->isEmpty() || $cur_post->f('tid') != $tid)
			return false;
		
		return true;
	}
	
	#------------------------------------------------
	# Index methods
	#------------------------------------------------
	
	/*
	@function splitWords
	
	Supprime d'une chaine de caractre les doublons et les mots 
	ne pas indexer dans les index de recherche.
	
	@param string text		la chaine de caractres
	*/
	function splitWords($text)
	{
		global $puntal;
		static $noise_match, $noise_replace, $stopwords;
	
		if (empty($noise_match))
		{
			$noise_match = array('[quote', '[code', '[url', '[img', '[email', '[color', '[colour', '[acronym', '[nospam', 'quote]', 'code]', 'url]', 'img]', 'email]', 'color]', 'colour]', 'acronym]', 'nospam]', '^', '$', '&', '(', ')', '<', '>', '`', '\'', '"', '|', ',', '@', '_', '?', '%', '~', '+', '[', ']', '{', '}', ':', '\\', '/', '=', '#', ';', '!', '*');
			$noise_replace = array('',       '',      '',     '',     '',       '',       '',          '',       '',        '',       '',     '',     '',       '',       '',       '',        '',         '',     ' ', ' ', ' ', ' ', ' ', ' ', ' ', '',  '',   ' ', ' ', ' ', ' ', '',  ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '' ,  ' ', ' ', ' ', ' ', ' ', ' ');
			
			$stopwords = (array)@file(PUN_ROOT.'lang/'.$puntal->user['language'].'/stopwords.txt');
			$stopwords = array_map('trim', $stopwords);
		}
	
		// Clean up
		$patterns[] = '#&[\#a-z0-9]+?;#i';
		$patterns[] = '#\b[\w]+:\/\/[a-z0-9\.\-]+(\/[a-z0-9\?\.%_\-\+=&\/~]+)?#';
		$patterns[] = '#\[\/?[a-z\*=\+\-]+(\:?[0-9a-z]+)?:[a-z0-9]{10,}(\:[a-z0-9]+)?=?.*?\]#';
		$text = preg_replace($patterns, ' ', ' '.strtolower($text).' ');
	
		// Filter out junk
		$text = str_replace($noise_match, $noise_replace, $text);
	
		// Strip out extra whitespace between words
		$text = trim(preg_replace('#\s+#', ' ', $text));
	
		// Fill an array with all the words
		$words = explode(' ', $text);
	
		if (!empty($words))
		{
			while (list($i, $word) = @each($words))
			{
				$words[$i] = trim($word, '.');
				$num_chars = strlen($word);
	
				if ($num_chars < 3 || $num_chars > 20 || in_array($word, $stopwords))
					unset($words[$i]);
			}
		}
	
		return array_unique($words);
	}
	
	
	/*
	@function updateSearchIndex
	
	Met  jour les index de recherche avec le contenu de post_ids
	
	@param string post_ids		Identifiant du post
	*/
	function updateSearchIndex($mode, $post_id, $message, $subject = null)
	{
		global $puntal;
	
		// Split old and new post/subject to obtain array of 'words'
		$words_message = punbb::splitWords($message);
		$words_subject = ($subject) ? punbb::splitWords($subject) : array();
	
		if ($mode == 'edit')
		{
			$row = $puntal->con->select('SELECT w.id, w.word, m.subject_match FROM '.DB_PREFIX.'search_words AS w INNER JOIN '.DB_PREFIX.'search_matches AS m ON w.id=m.word_id WHERE m.post_id='.$post_id) or $puntal->err->fatalError($puntal->con->error());
	
			// Declare here to stop array_keys() and array_diff() from complaining if not set
			$cur_words['post'] = array();
			$cur_words['subject'] = array();
	
			while ($row->fetch())
			{
				$match_in = ($row->f(2)) ? 'subject' : 'post';
				$cur_words[$match_in][$row->f(1)] = $row->f(0);
			}
			
			unset($row);
			
			$words['add']['post'] = array_diff($words_message, array_keys($cur_words['post']));
			$words['add']['subject'] = array_diff($words_subject, array_keys($cur_words['subject']));
			$words['del']['post'] = array_diff(array_keys($cur_words['post']), $words_message);
			$words['del']['subject'] = array_diff(array_keys($cur_words['subject']), $words_subject);
		}
		else {
			$words['add']['post'] = $words_message;
			$words['add']['subject'] = $words_subject;
			$words['del']['post'] = array();
			$words['del']['subject'] = array();
		}
	
		unset($words_message);
		unset($words_subject);
	
		// Get unique words from the above arrays
		$unique_words = array_unique(array_merge($words['add']['post'], $words['add']['subject']));
	
		if (!empty($unique_words))
		{
			$row = $puntal->con->select('SELECT id, word FROM '.DB_PREFIX.'search_words WHERE word IN('.implode(',', preg_replace('#^(.*)$#', '\'\1\'', $unique_words)).')') or $puntal->err->fatalError($puntal->con->error());
	
			$word_ids = array();
			while ($row->fetch())
				$word_ids[$row->f(1)] = $row->f(0);
			
			unset($row);
	
			$new_words = array_diff($unique_words, array_keys($word_ids));
			unset($unique_words);
	
			if (!empty($new_words))
			{
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'search_words (word) VALUES'.implode(',', preg_replace('#^(.*)$#', '(\'\1\')', $new_words))) or $puntal->err->fatalError($puntal->con->error());
			}
	
			unset($new_words);
		}
	
		// Delete matches (only if editing a post)
		while (list($match_in, $wordlist) = @each($words['del']))
		{
			$subject_match = ($match_in == 'subject') ? 1 : 0;
	
			if (!empty($wordlist))
			{
				$sql = '';
				while (list(, $word) = @each($wordlist))
					$sql .= (($sql != '') ? ',' : '').$cur_words[$match_in][$word];
	
				$puntal->con->execute('DELETE FROM '.DB_PREFIX.'search_matches WHERE word_id IN('.$sql.') AND post_id='.$post_id.' AND subject_match='.$subject_match) or $puntal->err->fatalError($puntal->con->error());
			}
		}
	
		// Add new matches
		while (list($match_in, $wordlist) = @each($words['add']))
		{
			$subject_match = ($match_in == 'subject') ? 1 : 0;
	
			if (!empty($wordlist))
				$puntal->con->execute('INSERT INTO '.DB_PREFIX.'search_matches (post_id, word_id, subject_match) SELECT '.$post_id.', id, '.$subject_match.' FROM '.DB_PREFIX.'search_words WHERE word IN('.implode(',', preg_replace('#^(.*)$#', '\'\1\'', $wordlist)).')') or $puntal->err->fatalError($puntal->con->error());
		}
	
		unset($words);
	}
	
	
	/*
	@function stripSearchIndex
	
	Supprime les index des mots indexs dans post_ids
	
	@param string post_ids		identifiants des posts spars par des virgules
	*/
	function stripSearchIndex($post_ids)
	{
		global $puntal;
	
		$row = $puntal->con->select('SELECT word_id FROM '.DB_PREFIX.'search_matches WHERE post_id IN('.$post_ids.') GROUP BY word_id') or $puntal->err->fatalError($puntal->con->error());
		
		if (!$row->isEmpty())
		{
			$word_ids = '';
			while ($row->fetch())
				$word_ids .= ($word_ids != '') ? ','.$row->f(0) : $row->f(0);
			
			unset($row);
	
			$row = $puntal->con->select('SELECT word_id FROM '.DB_PREFIX.'search_matches WHERE word_id IN('.$word_ids.') GROUP BY word_id HAVING COUNT(word_id)=1') or $puntal->err->fatalError($puntal->con->error());
	
			if (!$row->isEmpty())
			{
				$word_ids = '';
				while ($row->fetch())
					$word_ids .= ($word_ids != '') ? ','.$row->f(0) : $row->f(0);
	
				$puntal->con->execute('DELETE FROM '.DB_PREFIX.'search_words WHERE id IN('.$word_ids.')') or $puntal->err->fatalError($puntal->con->error());
			}
		}
	
		$puntal->con->execute('DELETE FROM '.DB_PREFIX.'search_matches WHERE post_id IN('.$post_ids.')') or $puntal->err->fatalError($puntal->con->error());
		
		return true;
	}
	
	
	#------------------------------------------------
	# Utils
	#------------------------------------------------
	
	/**
	@function updateForum
	 
	Met  jour nombre de messages, nombre de sujet, infos dernier message
	 
	@param	integer	forum_id		ID du forum qui doit tre mis  jour
	@return boolean
	*/
	function updateForum($forum_id)
	{
		global $puntal;
		
		$totaux = $puntal->con->select('SELECT COUNT(id) as num_topics, SUM(num_replies) as num_posts FROM '.DB_PREFIX.'topics WHERE moved_to IS NULL AND forum_id='.$forum_id) or $puntal->err->fatalError($puntal->con->error());
		
		$num_topics = $totaux->f('num_topics'); 
		$num_posts = $totaux->f('num_posts');
		
		$num_posts = $num_posts + $num_topics;	// $num_posts is only the sum of all replies (we have to add the topic posts)
	
		$infos = $puntal->con->select('SELECT last_post, last_post_id, last_poster FROM '.DB_PREFIX.'topics WHERE forum_id='.$forum_id.' AND moved_to IS NULL ORDER BY last_post DESC LIMIT 1') or $puntal->err->fatalError($puntal->con->error());
		if (!$infos->isEmpty())		// There are topics in the forum
		{
			$puntal->con->execute('UPDATE '.DB_PREFIX.'forums SET num_topics='.(integer)$num_topics.', num_posts='.(integer)$num_posts.', last_post='.(integer)$infos->f('last_post').', last_post_id='.(integer)$infos->f('last_post_id').', last_poster=\''.$puntal->con->escapeStr($infos->f('last_poster')).'\' WHERE id='.$forum_id) or $puntal->err->fatalError($puntal->con->error());
		}
		else	// There are no topics
			$puntal->con->execute('UPDATE '.DB_PREFIX.'forums SET num_topics=0, num_posts=0, last_post=NULL, last_post_id=NULL, last_poster=NULL WHERE id='.$forum_id) or $puntal->err->fatalError($puntal->con->error());
		
		return true;
	}
	
	
    /**
    @function xhtml2Bbcode
	@author Nicolas (nicolas2k10 sur punbb.fr)
    
    Transforme les balises XHTML en BBcode et 
    supprime celles qui n'ont pas d'quivalent.
    
    @param string str    Chane  convertir
    @return string
    */
    function xhtml2Bbcode($str)
    {
		# Valeur non-vide (exemple> src="'.$value.'" matchera src="qqchose")
        $value = '[^"]+';
		
		# 0 ou + paramtres et/ou espacements avant UN AUTRE PARAMETRE
        $attr = '(?:\s|[a-zA-Z-]+="'.$value.'")*';
		
		# Idem pour du CSS (exemple> text-decoration: none;)
        $attr_css_not_color = '(?:\s|(?!color)[a-zA-Z-]+\s*\:\s*[^;]*;)*';
		
		# 0 ou + paramtres et/ou espacements avant le > final
        $attr_avant_crochet = '[^>]*';
		
		# 0 ou + paramtres et/ou espacements avant le / (pour les balises seules)
        $attr_avant_slash = '[^/]*';
		
		# Espacements (retours de ligne, tabulations, espaces, etc.)
        $blank = '\s+';
		
		# Format des adresses e-mail
        $format_email = '\w+@\w+\.[a-z]{2,4}'; 


       /* 1> 
			- Pour les lments dont on A BESOIN DE CAPTURER LES ATTRIBUTS 
				COMME src="..." href="..." pour le remplacement
            - Pour les exceptions (tel que la balise "p" qui n'est pas convertie
			  en BBCode)
		*/
        $pattern = array(
            '#<span'.$attr.'style="'.$attr_css_not_color.'color\s*\:\s*([a-zA-Z-]+|\#?[[:xdigit:]]{6})\b[^"]*">(.*)</span>#Us',
            '#<a'.$attr.'href="(?=mailto)mailto:('.$format_email.')"'.$attr_avant_crochet.'>(.*)</a>#Us',
            '#<a'.$attr.'href="(?!mailto)('.$value.')"'.$attr_avant_crochet.'>(.*)</a>#Us',
            '#<img'.$attr.'src="('.$value.')"'.$attr_avant_slash.'/>#Us',
            '#<li'.$attr_avant_crochet.'>(.*)</li>#Us',
            '#<p'.$attr_avant_crochet.'>(.*)</p>#Us'
        );
        $replace = array(
            '[color=$1]$2[/color]',
            '[email=$1]$2[/email]',
            '[url=$1]$2[/url]',
            '[img]$1[/img]',
            '* $1',
            "$1\n\n"
        );

        /* 2> 
			Pour tous les lements dont ON N'A PAS BESOIN DE CAPTURER LES
			ATTRIBUTS COMME src="..." href="..." pour le remplacement
		*/
        $xhtml  = array('h[1-6]', 'strong', 'em', 'blockquote');
        $bbcode = array('b', 'b', 'i', 'quote');

        $count  = count($xhtml);
        for ($i=0; $i<$count; $i++)
		{
			$pattern[] = '#<('.$xhtml[$i].')'.$attr_avant_crochet.'>(.*)</\1>#Us';
			$replace[] = '['.$bbcode[$i].']$2[/'.$bbcode[$i].']';
        }

        /* 3> 
			Rajout conditionnel PunMultimedia, pour ne remplacer les "object"
			que si PunMultimedia a t dtct
		*/
        if (file_exists(PUN_ROOT.'punmultimedia.php'))
		{
			// Avec largeur et hauteur
			$pattern[] = '#<object'.$attr.'(?:'.$blank.'type="'.$value.'")+'.$attr.'(?:'.$blank.'data="('.$value.')")+'.$attr.'(?:'.$blank.'width="([[:digit:]]{1,3})"'.$blank.'height="([[:digit:]]{1,3})")+'.$attr_avant_crochet.'>.*<param'.$blank.'name="(?:filename|movie|src|url|file)+"'.$blank.'value="'.$value.'"'.$blank.'/>.*</object>#Us';
			$replace[] = '[media width="$2" height="$3"]$1[/media]';

			// Sans largeur ni hauteur
			$pattern[] = '#<object'.$attr.'(?:'.$blank.'type="'.$value.'")+'.$attr.'(?:'.$blank.'data="('.$value.')")+'.$attr_avant_crochet.'>.*<param'.$blank.'name="(?:filename|movie|src|url|file)+"'.$blank.'value="'.$value.'"'.$blank.'/>.*</object>#Us';
			$replace[] = '[media]$1[/media]';
        }

        /* 4> 
			Pour permettre de traiter des pages entires 
			via une URL (supprime script, style, title, etc.)
		*/
        $pattern[] = '#<head>.*</head>#Us';
        $replace[] = '<head></head>';

        /* 5> 
			Action !
		*/
        $str = preg_replace($pattern, $replace, $str);

        // Espacements, retours de ligne et caracres spciaux
        // le 21/08/2007 ajout de la gestion de &quot;
        $pattern = array('<br />', '    ', '	', '&amp;', '&lt;', '&gt;' , '&quot;');
        $replace = array("\n", "\t", "\t", '&', '<', '>','"');
        $str = str_replace($pattern, $replace, $str);

        return strip_tags($str);
    }

}

?>