<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

/*
	 DEBUG MODE
 
	 0 : disable
	 1 : simple debug, show execution time and queries numbers
	 2 : advanced debug, show simple debug + all detailled queries
*/
define('DEBUG',2);



#------------------------------------------------------------------------
# 			DO NOT CHANGE ANYTHING AFTER THIS LINE
#------------------------------------------------------------------------

define('PT_VERSION', '2.1.0');

define('PUNTAL', 1);

if (DEBUG > 0)
{
	# Record the start time
	list($usec, $sec) = explode(' ', microtime());
	$pt_start = ((float)$usec + (float)$sec);
	
	# Full error reporting
	@ini_set('display_errors','on');
	error_reporting(E_ALL);
}

# Unset any variables instantiated as a result of register_globals being enabled
if (ini_get('register_globals'))
{
	function unregister_globals()
	{
		if (isset($_REQUEST['GLOBALS']) || isset($_FILES['GLOBALS']))
			exit('Tut tut pouet pouet');
		
		// Variables that shouldn't be unset
		$no_unset = array('GLOBALS', '_GET', '_POST', '_COOKIE', '_REQUEST', '_SERVER', '_ENV', '_FILES');
	
		// Remove elements in $GLOBALS that are present in any of the superglobals
		$input = array_merge($_GET, $_POST, $_COOKIE, $_SERVER, $_ENV, $_FILES, isset($_SESSION) && is_array($_SESSION) ? $_SESSION : array());
		foreach ($input as $k => $v)
		{
			if (!in_array($k, $no_unset) && isset($GLOBALS[$k]))
			{
				unset($GLOBALS[$k]);
				unset($GLOBALS[$k]);	// Double unset to circumvent the zend_hash_del_key_or_index hole in PHP <4.4.3 and <5.1.4
			}
		}
	}
	unregister_globals();
}

# Strip slashes from GET/POST/COOKIE (if magic_quotes_gpc is enabled)
if (get_magic_quotes_gpc())
{
	function stripslashes_array($array)
	{
		return is_array($array) ? array_map('stripslashes_array', $array) : stripslashes($array);
	}

	$_GET = stripslashes_array($_GET);
	$_POST = stripslashes_array($_POST);
	$_COOKIE = stripslashes_array($_COOKIE);
}


# Load the Puntal advanced configuration
require dirname(__FILE__).'/puntal_config.php';	

# Get the required files
require dirname(__FILE__).'/classes/error.php';
require dirname(__FILE__).'/classes/mysql.php';
require dirname(__FILE__).'/classes/settings.php';
require dirname(__FILE__).'/classes/cookie.session.php';
require dirname(__FILE__).'/classes/puntal.php';
require dirname(__FILE__).'/classes/modules.php';
require dirname(__FILE__).'/classes/blocks.php';
require dirname(__FILE__).'/libs/l10n.php';
require dirname(__FILE__).'/libs/html.php';
require dirname(__FILE__).'/libs/date.php';

# Load the PunBB configuration
if (file_exists(dirname(__FILE__).'/config.php')) {
	require dirname(__FILE__).'/config.php';
}
if (!defined('PUN'))
{
	header('Content-Type: text/plain');
	echo 
	'[fr] Le fichier "config.php" n\'existe pas ou est endommag. '."\n".
	'Veuillez installer Puntal. Ou si Puntal est dj install, '."\n".
	'veuillez copier le fichier "config.php" de PunBB dans le '."\n".
	'rpertoire /inc de Puntal.'."\n\n".
	'[en] The file "config.php" does not exist or is damaged. '."\n".
	'Please install Puntal. Or if Puntal is already installed, '."\n".
	'please copy the file "config.php" of PunBB in the /inc '."\n".
	'directory of Puntal.';
	exit;
}

# If a cookie name is not specified in config.php, we use the default (punbb_cookie)
if (empty($cookie_name)) $cookie_name = 'punbb_cookie';

# Define a few commonly used constants
define('PUN_UNVERIFIED', 32000);
define('PUN_ADMIN', 1);
define('PUN_MOD', 2);
define('PUN_GUEST', 3);
define('PUN_MEMBER', 4);
define('DB_PREFIX', $db_prefix);

# Gestionnaire d'erreur
$error = new error();

# Connect
$con = new connection($db_username,$db_password,$db_host,$db_name);

# Get the Puntal settings
$pt_settings = new settings($con, DB_PREFIX.'pt_setting');
$pt_settings->export();

# Set commons paths & URLs if not defined in puntal_config.php
if (!defined('PUNTAL_ROOT')) define('PUNTAL_ROOT', dirname(__FILE__).'/../');
if (!defined('PT_INC_PATH')) define('PT_INC_PATH', PUNTAL_ROOT.'inc/');
if (!defined('PT_CACHE_PATH')) define('PT_CACHE_PATH', PUNTAL_ROOT.'cache/');
if (!defined('PT_MODULES_PATH')) define('PT_MODULES_PATH', PUNTAL_ROOT.'modules/');
if (!defined('PT_THEMES_PATH')) define('PT_THEMES_PATH', PUNTAL_ROOT.'themes/');
if (!defined('PT_THEMES_URL')) define('PT_THEMES_URL', pt_portal_path.'themes/');
if (!defined('PT_PUBLIC_DIR')) define('PT_PUBLIC_DIR', PUNTAL_ROOT.'public/');
if (!defined('PT_PUBLIC_PATH')) define('PT_PUBLIC_PATH', pt_portal_path.'public/');

# Set our own PUN_ROOT
if (!defined('PUN_ROOT')) define('PUN_ROOT', $_SERVER['DOCUMENT_ROOT'].pt_forum_url);
	
# Set our own REQUEST_URI if not set
if (empty($_SERVER['REQUEST_URI']))
{
	if (pt_url_scan == 'path_info')
		$_SERVER['REQUEST_URI'] = pt_portal_url.(!empty($_SERVER['PATH_INFO']) ? substr($_SERVER['PATH_INFO'], 1) : '');
	else
		$_SERVER['REQUEST_URI'] = pt_portal_url.(!empty($_SERVER['QUERY_STRING']) ? '?'.$_SERVER['QUERY_STRING'] : '');	
}

# Default templates set
if (!defined('PT_DEFAULT_THEME')) define('PT_DEFAULT_THEME', 'puntal');

# Init l10n
l10n::init();

# Get the permissions array
$pt_perms = unserialize(pt_perms);

# Go, go, go !
$puntal = new puntal($con,$error);

# Loading Puntal locales
l10n::set(dirname(__FILE__).'/../locales/'.$puntal->user['language'].'/puntal');



############################################################
############################################################
function mydebug($datas,$return=false,$exit=false)
{
	$str = '<pre>'.var_export($datas,true).'</pre>';
	if ($return) return $str;
	echo $str;
	if ($exit) exit;
}
############################################################
############################################################
?>