<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

$severity = '';
$status = '';
$show_filters = false;

if (!empty($_REQUEST['severity']))
{
	$severity = intval($_REQUEST['severity']);
	$show_filters = true;
}

if (!empty($_REQUEST['status']))
{
	$status = intval($_REQUEST['status']);
	$show_filters = true;
}

if ($status < 1 || $status > 8)
	$status = '< 7';
else 
	$status = '= '.$status;


/************************************************************************/

// Fonction d'affichage des lignes de bugs
function bugs_line($data,$i)
{
	$td_class = ($i%2 == 0) ? 'even' : 'odd';
	$sev_class = ' sev_'.$data['bug_severity'];
	
	$comments = '';
	if ($data['bug_tid'] > 0)
	{
		if ($data['num_replies'] > 1)
			$comments = sprintf(__('%s comments'), $data['num_replies']);
		else
			$comments = sprintf(__('%s comment'), $data['num_replies']);
	}
		
	return 
	'<td class="'.$td_class.'" scope="row"><a href="module.php?id=bugtracker&amp;action=3&amp;bid='.$data['bug_id'].'">'.html::escapeHTML($data['bug_title']).'</a></td>'.
	'<td class="'.$td_class.'">'.$data['bug_component'].'</td>'.
	'<td class="'.$td_class.'"><a href="'.pt_forum_url.'profile.php?id='.$data['bug_reporter'].'">'.html::escapeHTML($data['bug_username']).'</a></td>'.
	'<td class="'.$td_class.'">'.$comments.'</td>'.
	'<td class="'.$td_class.'">'.$data['bug_percent'].'%</td>'.
	'<td class="'.$td_class.'">'.$data['bug_version'].'</td>'.
	'<td class="'.$td_class.'">'.__('stat_'.$data['bug_status']).'</td>'.
	'<td class="'.$td_class.$sev_class.'">'.__('sev_'.$data['bug_severity']).'</td>';
}


/************************************************************************/

require PT_INC_PATH.'/classes/multipage.php';

$nb_per_page = 20;
$nb_bugs = $bugtracker->getTotalAllBug($severity,$status);
$max_pages = ceil($nb_bugs/$nb_per_page);

$env = (!empty($_GET['env']) && $_GET['env'] <= $max_pages) ? $_GET['env'] : 1;

$rs = $bugtracker->getBugs('',$severity,$status,'B.bug_severity DESC',(($env-1)*$nb_per_page).','.$nb_per_page);

$lum = new multipage($env, 'bugs_line', $rs->getData(), $nb_bugs, $nb_per_page);

$lum->setOption('html_block', 
	'<table class="bugs" cellpadding="0" cellspacing="0">
		<thead>
			<tr>
				<th scope="col">'.__('Bug').'</th>
				<th scope="col">'.__('Component').'</th>
				<th scope="col">'.__('Reporter').'</th>
				<th scope="col">'.__('Discuss').'</th>
				<th scope="col">'.__('Progress').'</th>
				<th scope="col">'.__('Version').'</th>
				<th scope="col">'.__('Status').'</th>
				<th scope="col">'.__('Severity').'</th>
			</tr>
		</thead>
		<tbody>%s</tbody>
	</table>'
);

$lum->setOption('html_row','<tr>%s</tr>');
$lum->setOption('html_cell','%s');

$lum->setOption('html_links','<p>'.__('Page(s):').' %s</p>');
$lum->setOption('html_cur_page','<strong>%s</strong>');

$lum->setOption('html_prev','&lt; '.__('Previous'));
$lum->setOption('html_next',__('Next').' &gt;');
$lum->setOption('html_prev_grp','...');
$lum->setOption('html_next_grp','...');

$lum->setOption('html_empty','<p><strong>'.__('No bug').'</strong></p>');

$severity_array = array();
$severity_array['&nbsp;'] = '';
for ($i=1; $i<6; $i++)
	$severity_array[__('sev_'.$i)] = $i;
	
$status_array = array();
$status_array['&nbsp;'] = '';
for ($i=1; $i<9; $i++)
	$status_array[__('stat_'.$i)] = $i;

# CSS & JS
$head = 
'<style type="text/css">
<!--
table.bugs {
	width: 100%;
	border: none;
	padding: 0;
	margin: 0;
}
table.bugs td, table.bugs th {
	padding: 3px 0;
	margin: 0;
}
table.bugs td.even {
	background: #eee;
}
table.bugs td.odd {
	background: #fff;
}

table.bugs td.sev_1, table.bugs td.sev_2, table.bugs td.sev_3, 
table.bugs td.sev_4, table.bugs td.sev_5 { padding-left: 5px; }

table.bugs td.sev_1 { background-color: #fff5dd; }
table.bugs td.sev_2 { background-color: #ecdbb7; }
table.bugs td.sev_3 { background-color: #f5d5c6; }
table.bugs td.sev_4 { background-color: #F7B390; }
table.bugs td.sev_5 { background-color: #f3a29b; }
-->
</style>';

if (!$show_filters)
	$head .= adminPage::jsLoad('js/filter-controls.js');

# Start page
adminPage::open(__('Bugs list').' - '.__('Bugtracker'),$head);
?>

<h2 class="first"><img src="index.php?pf=bugtracker/img/ico_bugtracker.png" alt="" class="icon" /> <?php echo __('Bugs list') ?></h2>

<?php # Filtres
if (!$show_filters)
	echo '<p><a id="filter-control" class="form-control" href="#">'.__('Filters').'</a></p>';
?>	

<form action="module.php" method="get" id="filters-form">
	<fieldset>
		<legend><?php echo __('Filters') ?></legend>
		<div class="three-cols">
			<p class="col">
				<label for="severity"><?php echo __('Severity') ?></label>
				<?php echo form::combo('severity', $severity_array, $severity) ?>
			</p>
			<p class="col">
				<label for="status"><?php echo __('Status') ?></label>
				<?php echo form::combo('status', $status_array, $status) ?>
			</p>
			<p class="col">
				<?php echo form::hidden('id','bugtracker')."\n".form::hidden('action',1); ?>
				<input class="submit" type="submit" value="<?php echo __('Display') ?>" />
			</p>
		</div>
	</fieldset>
</form>
<div class="clearb"></div>
<?php echo $lum->getLinks(); ?>
<?php echo $lum->getPage(); ?>
<?php echo $lum->getLinks(); ?>

<?php adminPage::close() ?>