<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

/**
	
	@brief bugtrackerManager module class
	
	<strong>FR</strong> La classe de gestion des bogues<br/>	
	<strong>EN</strong> The class of bugs management
*/
class bugtrackerManager
{
	var $pt;
	var $t;

	/**
	bugtrackerManager<br/>
	
	<strong>FR</strong> Constructeur<br/>
	<strong>EN</strong> Constructor
	
	@param puntal <strong>object</strong> puntal type object
	*/	
	function bugtrackerManager(&$pt)
	{
		$this->pt = $pt;
		$this->t = DB_PREFIX.'pt_bugs';
	}


	/**
	getBug<br/>
	
	<strong>FR</strong> Rcupration d'un bug<br/>
	<strong>EN</strong> get a bug
	
	@param id <strong>integer</strong> the id of the bug to get
	*/			
	function getBug($id)
	{
		return $this->getBugs($id);
	}
	
	/**
	getBugs<br/>
	
	<strong>FR</strong> Rcupration d'un ou de plusieurs bugs<br/>
	<strong>EN</strong> get one or more bugs
	
	@param id <strong>integer</strong> the id of the bug to get
	@param severity <strong>integer</strong> bug severity
	@param status <strong>integer</strong> bug status
	@param order_by <strong>string</strong> sort the bugs by ...
	@param limit <strong>string</strong> limit the number of bug to return
	@return	<strong>object</strong> recordset
	*/		
	function getBugs($id='',$severity='',$status='',$order_by='B.bug_severity DESC',$limit='')
	{		
		$req_plus = ' 1 ';
		if ($id != '') {
			$req_plus .= ' AND B.bug_id='.(integer) $id;
			# order by is unuseful when we get just one bug!
			$order_by 	= '';
			$status 	= '';
			$severity 	= '';
		}
		
		else {	
			if ($status != '')
			$req_plus .= ' AND B.bug_status'.$status;
			
			if ($severity != '')
				$req_plus .= ' AND B.bug_severity='.(integer) $severity;
		}

		$sql =  
		'SELECT B.bug_id, ' .
		'B.bug_title, ' .
		'B.bug_desc, ' .
		'B.bug_severity, ' .
		'B.bug_status, ' .
		'B.bug_percent, '.
		'B.bug_reporter, ' .
		'B.bug_version, ' .
		'B.bug_tid, ' .
		'B.bug_pid, ' .
		'B.bug_component, '.
		'U.username AS bug_username, '.
		'T.num_replies '.
		'FROM '.$this->t.' B '.
		'LEFT JOIN '.DB_PREFIX.'users U ON B.bug_reporter=U.id '.
		'LEFT JOIN '.DB_PREFIX.'topics T ON B.bug_tid=T.id '.
		'WHERE '.$req_plus.' ';
		
		if ($order_by != '')
			$sql .= 'ORDER BY '.$order_by;
	
		if ($limit != '')
			$sql = $sql.' LIMIT '.$limit;
		
		$rs = $this->pt->con->select($sql) or $this->pt->err->fatalError($this->pt->con->error());

		return $rs;
	}

	/**
	setBug<br/>
	
	<strong>FR</strong> Ajout ou modification d'un bug<br/>
	<strong>EN</strong> add or modifiy a bug
	
	@param title <strong>string</strong> bug title
	@param desc <strong>string</strong> bug description
	@param severity <strong>integer</strong> bug severity
	@param status <strong>integer</strong> bug status
	@param per_cent <strong>integer</strong> bug progress
	@param version <strong>string</strong> version of the 'thing' on which we meet the bug
	@param tid <strong>integer</strong> topic id to link to the forum
	@param pid <strong>integer</strong> post id to link to the forum
	@param id <strong>integer</strong> bug id (for modify purpose)
	@param component <strong>string</strong> component on which we meet the bug
	@return	<strong>object</strong> recordset
	*/		
	function setBug($title,$desc,$severity=3,$status=1,$per_cent=0,$version='',$tid=0,$pid=0,$id=0,$component)
	{
		require_once PT_INC_PATH."libs/punbb_parser.php";
		$desc = preparse_bbcode($desc,false);
		// update a bug
		if ($id > 0)
		{
			$updreq = 
			'UPDATE '.$this->t.' SET '.
			'bug_title=\''	.$this->pt->con->escapeStr($title).'\', '.
			'bug_desc=\''	.$this->pt->con->escapeStr($desc).'\', '.
			'bug_severity='	.(integer) $severity.', '.
			'bug_status='	.(integer) $status.', '.
			'bug_percent='	.(integer) $per_cent.', '.
			'bug_version=\''.$this->pt->con->escapeStr($version).'\', '.
			'bug_tid='		.(integer) $tid.', '.
			'bug_pid='		.(integer) $pid.', '.
			'bug_component=\''.$this->pt->con->escapeStr($component).'\' '.
			'WHERE bug_id='.(integer) $id;
			
			$this->pt->con->execute($updreq) or $this->pt->err->fatalError($this->pt->con->error());
		}
		// add a bug
		else {
			$addreq = 
			'INSERT INTO '.$this->t.' '.
			
			'(bug_title, ' .
			'bug_desc, ' .
			'bug_severity, ' .
			'bug_status, ' .
			'bug_reporter, ' .
			'bug_version, ' .
			'bug_tid, ' .
			'bug_pid, ' .
			'bug_component) '.
			
			'VALUES('.
			'\''.$this->pt->con->escapeStr($title).'\', '.
			'\''.$this->pt->con->escapeStr($desc).'\', '.
			(integer) $severity.', '.
			(integer) $status.', '.
			(integer) $this->pt->user['id'].', '.
			'\''.$this->pt->con->escapeStr($version).'\', '.
			(integer) $tid.', '.
			(integer) $pid.','.
			'\''.$this->pt->con->escapeStr($component).'\''.
			')';
			
			$this->pt->con->execute($addreq) or $this->pt->err->fatalError($this->pt->con->error());
		}
			
		return true;
	}


	/**
	getTotalAllBug<br/>
	
	<strong>FR</strong> Retourne le nombre total de bugs, tout statut/svrit  confondus<br/>
	<strong>EN</strong> Return the total number of bugs, all status and severity included
	
	@param severity <strong>integer</strong> bug severity
	@param status <strong>integer</strong> bug status
	@return	<strong>integer</strong> total of bugs
	*/
	function getTotalAllBug($severity='',$status='')
	{
		$req_plus = ' WHERE 1';
		
		if ($severity != '')
			$req_plus .= ' AND bug_severity='.(integer) $severity;
			
		if ($status != '')
			$req_plus .= ' AND bug_status'.$status;
		
		$req = 'SELECT COUNT(bug_id) AS total_bugs FROM '.$this->t.$req_plus;
		
		$result = $this->pt->con->select($req) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $result->f('total_bugs');	
	}


	/**
	searchEntry<br/>
	
	<strong>FR</strong> Recherche d'un bug selon des mots cl et renvoie un recordset.<br/>
	<strong>EN</strong> Seach of bug from keywords and return the recordset.
	
	@param w <strong>string</strong> keyword
	@param limit <strong>string</strong> limit the number of results to be displayed
	@return	<strong>object</strong> recordset
	*/
	function searchEntry($w,$limit='')
	{
		$strip = array('+',')','(',',','\'');
		$w = strtolower($w);
		$w = str_replace($strip,' ',$w);
		$w = preg_replace('/( )+/',' ',$w);
		
		$reqPlus = '';
		foreach(explode(' ',$w) as $v)
		{
			$reqPlus .= 
			' AND (LOWER(bug_title) LIKE \'%'.$this->pt->con->escapeStr($v).'%\' OR '.
			'LOWER(bug_desc) LIKE \'%'.$this->pt->con->escapeStr($v).'%\') ';
		}
		
		$sql = 
		'SELECT * '.
		'FROM '.$this->t.
		' WHERE 1=1 '.
		$reqPlus.
		' ORDER BY bug_severity DESC';
        
        if ($limit != '')
			$sql = $sql.' LIMIT '.$limit;        
            
		$rs = $this->pt->con->select($sql) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $rs;
	}
	
}

?>
