<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

require_once PT_INC_PATH."libs/punbb_parser.php";

# Bugtracker
require dirname(__FILE__).'/inc/bugtracker.class.php';
$bugtracker = new bugtrackerManager($puntal);

# Template
require dirname(__FILE__).'/inc/tpl_functions.php';
$pt_template_file = '/bugtracker/main.php';

# Initialisations
$allowed_groups = unserialize(pt_mod_bugtracker_groups);

$bt_mode 		= 'default';
$neo_bug 		= '';
$neo_desc 		= '';
$neo_severity 	= 3;
$neo_version 	= '';
$neo_component 	= '';
$req_username 	= '';
$req_email 		= '';

# Titre page
$pt_custom_title_page[] = __('Bugtracker');

/**
	Parcours des arguments pour dfinir le mode
	et actions en fonction de ce mode
*/
if (!empty($pt_args[1]))
{
	# ajout d'un bug
	if ($pt_args[1] == BT_URL_ADD && ($puntal->user['is_admin'] || in_array($puntal->user['g_id'], $allowed_groups)))
	{
		$bt_mode = 'ajouter';
		$pt_custom_title_page[] = __('Add a bug');
		
		if (isset($_POST['form_sent']) && $puntal->checkToken())
		{
			$errors = array();
			
			$neo_bug 		= trim($_POST['neo_bug']);
			$neo_desc 		= trim($_POST['neo_desc']);
			$neo_severity 	= intval($_POST['neo_severity']);
			$neo_version 	= trim($_POST['neo_version']);
			$neo_component 	= trim($_POST['neo_component']);

			if ($puntal->user['is_guest'])
			{
				$req_username = trim($_POST['req_username']);
				$req_email = trim($_POST['req_email']);
			}
			else {
				$req_username = $puntal->user['username'];
				$req_email = $puntal->user['email'];
			}
			
			if ($neo_bug == '')
				$errors[] = __('You must give a bug title.');
			
			if ($neo_desc == '')
				$errors[] = __('You must give a bug description.');
			
			$t_infos = array('tid' => 0, 'pid' => 0);
						
			if (empty($errors) && !isset($_POST['preview']) && pt_mod_bugtracker_forum_id > 0)
			{
				require PT_INC_PATH.'libs/punbb.php';
				
				if (($t_infos = punbb::newTopic(pt_mod_bugtracker_forum_id,$neo_bug,$neo_desc,true,false,false,$req_username,$req_email,true)) === false)
				{
					foreach ($puntal->err->getError(false) as $err)
						$errors[] = implode('<br />',$err);
				}
			}
			if (empty($errors) && !isset($_POST['preview']))
			{				
				if ($bugtracker->setBug($neo_bug,$neo_desc,$neo_severity,1,0,$neo_version,$t_infos['tid'],$t_infos['pid'],0,$neo_component))
					$puntal->redirect($puntal->url('bugtracker'), __('Bug added.'));
				else {
					foreach ($puntal->err->getError(false) as $err)
						$errors[] = implode('<br />',$err);
				}
			}
		} else $puntal->setToken();
	}
	
	# vue d'un bug
	if ($pt_args[1] == BT_URL_BUG && !empty($pt_args[2]) and is_numeric($pt_args[2]))
	{
		$bt_mode = 'bug';
		
		$bug 					= $bugtracker->getBug($pt_args[2]);
		$pt_custom_title_page[] = $bug->f('bug_title');
		
		if ($bug->f('bug_tid') > 0)
		{
			require_once PT_INC_PATH.'libs/punbb.php';
			if (($comments = punbb::getTopicPosts($bug->f('bug_tid'),'',$bug->f('bug_pid'))) === false)
				$puntal->err->fatalError();
			require_once PT_INC_PATH.'libs/punbb_parser.php';
		}
	}
}

// Si on doit afficher une liste de bugs
if ($bt_mode == 'default')
{
	$filter_box = '-1';
	# svrit ou statut demand ?
	if (!empty($_POST['filters']))
	{
		if (!empty($_POST['severity']))
			$pt_args[] = BT_URL_SEVERITY.$_POST['severity'];
		
		if (!empty($_POST['status']))
			$pt_args[] = BT_URL_STATUS.$_POST['status'];
		
		$filter_box = '1';
	}
	
	$env = 1;
	$severity = '';
	$status = '';
	foreach ($pt_args as $arg)
	{
		if (preg_match('!^'.BT_URL_PAGE.'([0-9]+)!',$arg,$match))
			$env = intval($match[1]);
		elseif (preg_match('!^'.BT_URL_SEVERITY.'([0-9]+)!',$arg,$match))
			$severity = intval($match[1]);
		elseif (preg_match('!^'.BT_URL_STATUS.'([0-9]+)!',$arg,$match))
			$status = intval($match[1]);
	}
		
	if ($severity < 1 || $severity > 5) 
		$severity = '';
	
	if ($status < 1 || $status > 8)
		$status = '< 7';
	else 
		$status = '= '.$status;

	# prpare mutlipage
	require PT_INC_PATH.'classes/multipage.php';
	$nb_bugs 	= $bugtracker->getTotalAllBug($severity,$status);
	$max_pages 	= ceil($nb_bugs/pt_mod_bugtracker_num_per_page);
		
	if ($env > $max_pages) 
		$env = 1;
	
	# get bugs
	$rs = $bugtracker->getBugs('',$severity,$status,'B.bug_severity DESC',(($env-1)*pt_mod_bugtracker_num_per_page).','.pt_mod_bugtracker_num_per_page);
	
	# construit multipage
	$lum = new multipage($env, 'bugsListLine', $rs->getData(), $nb_bugs, pt_mod_bugtracker_num_per_page,'','',true);
	
	# paramtres multipage
	$lum->setOption('html_block', bugsListBlock());
	
	$lum->setOption('html_row','<tr>%s</tr>');
	$lum->setOption('html_cell','%s');
	
	$lum->setOption('html_links','<p>'.__('Pages').' %s</p>');
	$lum->setOption('html_cur_page','<strong>%s</strong>');
	
	$lum->setOption('html_prev','&lt; '.__('Previous'));
	$lum->setOption('html_next',__('Next').' &gt;');
	$lum->setOption('html_prev_grp','...');
	$lum->setOption('html_next_grp','...');
	$lum->setOption('prefix_url',BT_URL_PAGE);
	
	$lum->setOption('html_empty','<p><strong>'.__('No bug').'</strong></p>');
}
?>