<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, InC., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

/**
	
	@brief downloadAdminManager module class
	
	<strong>FR</strong> La classe de gestion du module Tlchargements <strong>Cot Admin</stron><br/>	
	<strong>EN</strong> The class of management of the downloads <strong>on Admin Side</strong>
*/
class downloadAdminManager extends downloadManager
{
	var $cat_id = ''; 				// catgorie courante   

	/**
	downloadAdminManager<br/>
	
	<strong>FR</strong> Constructeur<br/>
	<strong>EN</strong> Constructor
	
	@param puntal <strong>object</strong> puntal type object
	*/
	function downloadAdminManager(&$puntal)
	{
		$this->pt = $puntal;
		$this->pt_dl = DB_PREFIX.'pt_downloads';
		$this->pt_dl_cat = DB_PREFIX.'pt_downloads_cat';
	}
	
	/**
	getDownloads
	
	<strong>FR</strong> Rcupration des tlchargements<br/>
	<strong>EN</strong> Get the downloads
	
	@return	rs <strong>object</strong> recordset
	*/
	function getDownloads($cat_id='',$status='',$order_by='C.dl_disp_position, D.dl_name',$limit='')
	{
		$req_plus = 'D.cat_id=C.id';
		
		if ($cat_id != '')
		{
			if (preg_match('/^[0-9]+$/',$cat_id))
				$req_plus .= ' AND C.id='.(integer) $cat_id;
			else
				$req_plus .= ' AND C.cat_url=\''.$this->pt->con->escapeStr($cat_id).'\'';
		}
		
		if ($status != '')
			$req_plus .= ' AND P.dl_enable='.(integer) $status;
		
		$sql = 
		'SELECT D.*, C.dl_cat_name '.
		'FROM '.$this->pt_dl.' as D, '.$this->pt_dl_cat.' as C '.
		'WHERE '.$req_plus.' '.
		'ORDER BY '.$order_by;
				
		if ($limit != '')
			$sql = $sql.' LIMIT '.$limit;
		
		$rs = $this->pt->con->select($sql) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $rs;
	}
	
	/**
	getDownloads
	
	<strong>FR</strong> Rcupration de toutes les infos d'un tlchargement<br/>
	<strong>EN</strong> Get all the ifnos for a given the download
	
	@param	id <strong>integer</strong> download ID
	@return	rs <strong>object</strong> recordset
	*/	
	function getDownload($id='')
	{
		$id = intval($id);
		if ($id < 1)
		{
			$this->pt->err->setError('Bad argument');
			return false;
		}
		
		$strReq =
		'SELECT P.*, C.dl_cat_name, C.dl_cat_url '.
		'FROM '.$this->pt_dl.' P '.
		'LEFT JOIN '.$this->pt_dl_cat.' C ON P.cat_id=C.id '.
		'WHERE P.id='.(integer) $id;
		
		$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());

		return $rs;
	}
	
	/**
	setDownload<br/>
	
	<strong>FR</strong> Ajout / modification d'un Download. <br/>
	<strong>EN</strong> Add / modify of a download.	
	
	@param	id <strong>integer</strong> download ID
	@param	name <strong>string</strong>		Download name
	@param	url <strong>string</strong>			'URLized' of the download name
	@param	filename <strong>string</strong>	filename
	@param	desc <strong>string</strong>		description of the download
	@param	count <strong>integer</strong>		number of downloads
	@param	date	<strong>string</strong> 	Download date
	@param	cat_id	<strong>integer</strong>	Category ID of the download
	@param	disp <strong>integer</strong>		Download position
	@param	home <strong>integer</strong>		Do we display the download in the download bloc ?
	@param	enable <strong>integer</strong>		Download Status
	@param	welcome <strong>string</strong>		Do we display the download on the welcome page of the download module ?
	@param	tid	 <strong>integer</strong>		topic id where is stored the post  
	@param	pid	 <strong>integer</strong>		post id where is stored the post
	
	@return	id <srtong>integer</strong> the id of the download
	*/
	function setDownload($id=0,$name='',$filename='',$desc='',$count=0,$date='',$cat_id=0,$disp=1,$home=1,$enable=2,$welcome=1,$tid=0,$pid=0)
	{
		$id 		= intval($id);
		$name 		= trim($name);
		$filename 	= trim($filename);
		$desc 		= trim($desc);
		$date 		= trim($date);
		$count	 	= intval($count);
		$cat_id 	= intval($cat_id);
		$disp 		= intval($disp);
		$home 		= intval($home);
		$enable 	= intval($enable);
		$welcome 	= intval($welcome);
		$tid 		= intval($tid);
		$pid 		= intval($pid);
		
		if ($name == '')
			$this->pt->err->setError(__('You must give a name for the download.'));
		
		if ($filename == '')
			$this->pt->err->setError(__('You must choose a filename for the download.'));
		
		if ($desc == '')
			$this->pt->err->setError(__('You must give a description for the download.'));				
		
		if ($date == '')
			$this->pt->err->setError(__('You must give a date for the download.'));		
		
		else {
			list($d_y,$d_m,$d_d) = split("[/.-]",$date);
			if (checkdate ($d_m,$d_d,$d_y) == false)
				$this->pt->err->setError(__('Invalid date - the date must be in YYYY-MM-DD form (Year-Month-Day).'));
		}
		
		if ($cat_id == '' || !$this->checkCat($cat_id) )
				$this->pt->err->setError(__('Category not exists'));
		
		# si il y a des erreurs
		if ($this->pt->err->hasError())
			return false;
		
		$name = $this->pt->secureString($name);
		
		# Ajout d'un Download
		if ($id == 0)
		{	
			// vrifiction que ce fichier n'est pas dj attribuer pour un autre tlchargement.
			$checked_dl = $this->checkDownload($filename) ;
			
			if  ($checked_dl->f('nb_files') > 0 && $checked_dl->f('dl_filename') != '')
			{
				$this->pt->err->setError(__('This file is already used by another download.'));
				return false;				
			}				
			else {
				$insReq = 
				'INSERT INTO '.$this->pt_dl.' '.
				'(dl_name,dl_filename,dl_desc,dl_count,dl_date,cat_id,'.
				'dl_disp_position,dl_home,dl_enable,dl_welcome) VALUES ('.
				'\''.$this->pt->con->escapeStr($name).'\', '.
				'\''.$this->pt->con->escapeStr($filename).'\', '.
				'\''.$this->pt->con->escapeStr($desc).'\', '.
				''.(integer) $count.', '.
				'\''. $date.'\', '.
				(integer) $cat_id.', '.
				(integer) $disp.', '.
				(integer) $home.', '.
				(integer) $enable.', '.
				(integer) $welcome.' '.
				') ';

				$this->pt->con->execute($insReq) or $this->pt->err->fatalError($this->pt->con->error());

				$id = $this->pt->con->getLastID();
				logIt('Downloads module : add download '.$name, 1);
				return $id;
			}
		}
		# Modification d'un Download
		else {
			$updReq = 
			'UPDATE '		.$this->pt_dl.' SET '.
			'dl_name=\''	.$this->pt->con->escapeStr($name).'\','.
			'dl_filename=\''.$this->pt->con->escapeStr($filename).'\', '.
			'dl_desc=\''	.$this->pt->con->escapeStr($desc).'\', '.
			'dl_count='		.(integer) $count.', '.
			'dl_date=\''	.$date.'\', '.
			'cat_id='		.(integer) $cat_id.', '.
			'dl_disp_position='.(integer) $disp.', '.
			'dl_home='		.(integer) $home.', '.
			'dl_enable='	.(integer) $enable.', '.
			'dl_welcome='	.(integer) $welcome.', '.
			'dl_tid='		.(integer) $tid.', '.
			'dl_pid='		.(integer) $pid.' '.
			'WHERE id='		.(integer) $id.' ';

			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

			logIt('Downloads module : update download '.$name, 1);
			return $id;
		}
	}
		
	/**
	delDownload<br/>

	<strong>FR</strong> Suppression d'un Download donn. Renvoie vrai en cas de succs. <br/>
	<strong>EN</strong> Remove a given download. Return true in case of success	
	
	@param	id <strong>integer</strong> download ID
	@return <strong>boolean</strong>	
	*/
	function delDownload($id)
	{		
		$delReq = "DELETE FROM ".$this->pt_dl." WHERE id = '".(integer)$id."' ";
		
		$this->pt->con->execute($delReq) or $this->pt->err->fatalError($this->pt->con->error());
		
		
		return true;
	}
	
	/**
	statusDownload<br/>

	<strong>FR</strong> Met un Download en ligne/hors ligne. Renvoie vrai en cas de succs.<br/>
	<strong>EN</strong> Update the status (online/offline) of a given download. Return true in case of success	
	
	@param	id <strong>integer</strong> download ID
	@param	status <strong>integer</strong> download status
	@return <strong>boolean</strong>	
	*/	
	function statusDownload($id,$status=2)
	{		
		$updReq = 
		'UPDATE '.$this->pt_dl.' '.
		'SET dl_enable='.(integer) $status.' '.
		'WHERE id='.(integer) $id.' ';
		
		$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

		return true;
	}

	/**
	checkDownload<br/>

	<strong>FR</strong> Vrification de l'existence d'un fichier dans la table.<br/>
	<strong>EN</strong> Update the status (online/offline) of a given download. Return true in case of success	
	
	@param	filename <strong>string</strong> filename	
	@return rs <strong>object</strong> recordset	
	*/		
	function checkDownload($filename='')
	{
		$filename = (string) $filename;
		if ($filename == '')
		{
			$this->pt->err->setError('Bad argument');
			return false;
		}
		
		$strReq =
		'SELECT COUNT(id) AS nb_files, dl_name, dl_filename '.
		'FROM '.$this->pt_dl.' '.
		'WHERE dl_filename=\''.$this->pt->con->escapeStr($filename).'\' '.
		'GROUP BY dl_filename';
		
		$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());

		return $rs;
	}
	
	/**
	setCat<br/>
	
	<strong>FR</strong> Ajout / modification d'une catgorie de tlchargement. Renvoie vrai en cas de succs.<br/>
	<strong>EN</strong> Add / modify of a download category. Return true in case of success.	
	
	
	@param	cat_name <strong>string</strong>	Label of the category
	@param	cat_url <strong>string</strong>			'URLized' of the category name
	@param	cat_desc <strong>string</strong>		description of the category
	@param	cat_id <strong>integer</strong> category ID
	
	@return	<strong>boolean</strong>
	*/	
	function setCat($cat_name='',$cat_url='',$cat_id=0)
	{
		$cat_name = trim($cat_name);
		$cat_url = trim($cat_url);
		
		if ($cat_name == '')
		{
			$this->pt->err->setError(__('Empty category title'));
			return false;
		}
		
		$cat_url = ($cat_url == '') ? $cat_name : $cat_url;
		$cat_url = $this->pt->str2url($cat_url);
		
		$cat_url = ucfirst(preg_replace('/^[0-9]{4,}/','',$cat_url));
		
		if ($cat_url == '')
		{
			$this->pt->err->setError(__('Empty or invalid category URL title'));
			return false;
		}
		
		$cat_name = $this->pt->secureString($cat_name);
		
		# Ajout d'une catgorie
		if ($cat_id == '')
		{
			$strReq = 'SELECT MAX(dl_disp_position) FROM '.$this->pt_dl_cat;
			
			$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());

			$max_ord = $rs->f(0);
			
			$insReq = 
			'INSERT INTO '.$this->pt_dl_cat.' '.
			'(dl_cat_name,dl_cat_url,dl_disp_position) VALUES ('.
			'\''.$this->pt->con->escapeStr($cat_name).'\', '.
			'\''.$this->pt->con->escapeStr($cat_url).'\', '.
			(integer)($max_ord+1).
			') ';

			$this->pt->con->execute($insReq) or $this->pt->err->setError($this->pt->con->error());

			logIt('Download module : create category '.$cat_name, 1);
			return true;
		}
		
		# Modification d'une catgorie
		else {
			if (!$this->checkCat($cat_id))
			{
				$this->pt->err->setError(__('Category not exists'));
				return false;
			}
			
			$updReq = 
			'UPDATE '.$this->pt_dl_cat.' SET '.
			'dl_cat_name=\''.$this->pt->con->escapeStr($cat_name).'\', '.
			'dl_cat_url=\''.$this->pt->con->escapeStr($cat_url).'\'  '.
			'WHERE id='.(integer) $cat_id.' ';
			
			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());
			
			logIt('Download module : update category '.$cat_name, 1);
			return true;
		}
	}
	

	/**
	delCat<br/>

	<strong>FR</strong> Suppression d'une catgorie donne. Renvoie vrai en cas de succs. <br/>
	<strong>EN</strong> Remove a given category. Return true in case of success	
	
	@param	cat_id <strong>integer</strong> category ID
	@return <strong>boolean</strong>	
	*/
	function delCat($cat_id)
	{
		if ($cat_id == '' || !$this->checkCat($cat_id))
		{
			$this->pt->err->setError(__('Category not exists'));
			return false;
		}
		
		$rs = $this->getCat($cat_id);
		
		if ($rs->f('nb_cat') > 0)
		{
			$this->pt->err->setError(__('Category not empty'));
			return false;
		}
		
		$cat_name = $rs->f('dl_cat_name');
		
		$delReq = 
		'DELETE FROM '.$this->pt_dl_cat.' '.
		'WHERE id='.(integer) $cat_id.' ';
		
		$this->pt->con->execute($delReq) or $this->pt->err->fatalError($this->pt->con->error());

		logIt('Download module : delete category '.$cat_name, 3);
		$this->reordCats();
		return true;
	}
	

	/**
	reordCats<br/>

	<strong>FR</strong> Rordonner les catgories. Cette mthode va remettre les catgorie dans
	l'ordre (selon les ID ou selon les noms si $byname est vrai).<br/>
	
	<strong>EN</strong> Sort the categories. This function will sort the categories in the order 
	(by ID or by Name)	
	
	@param check_url <strong>boolean</strong> 'URLized' of the titles
	@param byname <strong>boolean</strong>	sort by name
	*/	
	function reordCats($check_url=false,$byname=false)
	{
		$ordby = $byname ? 'cat_name' : 'dl_disp_position';
		
		$i = 0;
		$strReq = 
		'SELECT id, dl_cat_url '.
		'FROM '.$this->pt_dl_cat.' '.
		'ORDER BY '.$ordby.' ';
		
		$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());
		
		while (!$rs->EOF())
		{
			$reqPlus = '';
			if ($check_url && preg_match('/^[a-z]/',$rs->f('dl_cat_url')))
				$reqPlus = ',dl_cat_url=\''.ucfirst($rs->f('dl_cat_url')).'\' ';
			
			$updReq = 
			'UPDATE '.$this->pt_dl_cat.' SET '.
			'dl_disp_position='.(integer) $i.' '.
			$reqPlus.
			'WHERE id='.(integer) $rs->f('id');
			
			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());
			
			$i++;
			$rs->moveNext();
		}
		
		return true;
	}
	

	/**
	ordCats<br/>

	<strong>FR</strong> Ordonne les catgories en utilisant les valeurs du tableau ord. 
	La cl est l'identifiant de la catgorie et la valeur, l'ordre de celle-ci..<br/>
	
	<strong>EN</strong> Sort the categories. 
	
	@param ord <strong>array</strong> array containing the order of each category
	*/		
	function ordCats($ord)
	{
		if (!is_array($ord))
		{
			$GLOBALS['puntal']->setError('Bad argument');
			return false;
		}
		
		foreach ($ord as $k => $v)
		{
			$updReq = 
			'UPDATE '.$this->pt_dl_cat.' SET '.
			'dl_disp_position='.(integer) $k.' '.
			'WHERE id='.(integer) $v;
			
			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());
		}
		
		return true;
	}
		
	/**
	getDir<br/>
		
	<strong>FR</strong>	recupere les fichiers du repertoire de stockage<br/> 	
	<strong>EN</strong> get the file of the directory where they are stored

	@return	files <strong>array</strong> list of file
	*/	
	function getDir()
	{
		$the_path = PUNTAL_ROOT.pt_mod_downloads_path;

		// le repertoire existe-t-il ?
		if (!is_dir($the_path))
		{
			$this->pt->err->setError(sprintf(__('The directory %s does not exist.'),$the_path));
			return false;
		}
		else {
			$files = array();
			
			// ouverture du repertoire des telechargements
			$o = opendir($the_path);
			
			// parcours du repertoire des telecahrgements
			while (($file = readdir($o)) !== false)
			{
				// a-t-on trouve un fichier ?
				if (is_file($the_path."/".$file))
					$files[] = $file;
			}
			
			// fermeture du pointeur
			closedir($o);			
			
			// retour des fichiers de tlchargement
			return $files;
		}
	}


	/**
	getFileName<br/>
		
	<strong>FR</strong>
	recupere le nom du fichier<br/> 	
	
	<strong>EN</strong>
	get the filename<br/> 	
	
	@param	id <strong>integer</strong>		File ID
	@return	rs <strong>object</strong> recordset
	*/
	function getFileName($id='')
	{
		$id = intval($id);
		if ($id < 1)
		{
			$this->pt->err->setError('Bad argument');
			return false;
		}
		
		$strReq =
		'SELECT dl_filename '.
		'FROM '.$this->pt_dl.' '.
		'WHERE id='.(integer) $id.' ';
		
		$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());

		return $rs;
	}

	/**
	getCat<br/>
	
	<strong>FR</strong>
	'surchage' de la methode getCat de la classe downloadManager <br/> 
	
	le but etant d'afficher le nombre de fichiers (actifs ou non) d'une catgorie<br/>
	
	Slectionne une ou plusieurs catgories. La paramtre cat_id peut tre
	un entier, le champ <em>cat_id</em> sera alors pris en compte. S'il s'agit
	d'une chaine, la comparaison sera faite sur <em>cat_url</em>.<br/>
	
	compte galement les fichiers 'actif' d'une catgorie<br/>
	
	Si le paramtre cat_id est une chaine vide, toutes les catgories seront
	renvoyes dans le recordset (fonctionnement par dfaut).<br/>
	
	La mthode renvoie false en cas d'erreur.<br/>
	
	<strong>EN</strong>
	overwrite the function getCat of the class  downloadManager <br/>
	
	the goal is the display the number of file (activated or not) for one categorie<br/>
	
	Select one or several categories. The cat_id parameter may be an integer, the field <em>cat_id</em> will be used.
	If it's a string then the search will be done on <em>cat_url</em>.<br/>
	
	Count also the <em>activated</em> files for a category<br/>
	
	If the parameter cat_id is an empty string, all the categories will be return in the recordset (by default) <br/>
	
	The method returne false in case of error.<br/>	
	
	@param	cat_id <strong>mixed</strong>			Category ID
	@param	order_by <strong>string</strong>		field that sort the results
	@return	rs <strong>object</strong> recordset
	*/	
	function getCat($cat_id='',$order_by='C.dl_disp_position')
	{
		$reqPlus = '1';
		
		if ($cat_id != '')
		{
			if (preg_match('/^[0-9]+$/',$cat_id))
				$reqPlus .= ' AND C.id='.(integer)$cat_id;
			else
				$reqPlus .= ' AND C.dl_cat_url=\''.$this->pt->con->escapeStr($cat_id).'\'';
		}
		
		$strReq = 'SELECT C.id, C.dl_cat_name, C.dl_cat_url, '.
		'count(F.id) AS nb_files '.
		'FROM '.$this->pt_dl_cat.' C '.
		"LEFT JOIN ".$this->pt_dl." F ON C.id = F.cat_id ".
		'WHERE '.$reqPlus.' '.
		'GROUP BY C.id '.
		'ORDER BY '.$this->pt->con->escapeStr($order_by); 
		
		$rs = $this->pt->con->select($strReq) or $this->pt->err->fatalError($this->pt->con->error());

		return $rs;
	}


}

?>