<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

/**
	La classe de gestion du lexique.
*/
class lexiqueManager
{
	var $pt;
	var $t;
	
	/*
	@function lexique
	
	"Constructeur"
	
	@param object pt	objet type puntal
	*/
	function lexiqueManager(&$pt)
	{
		$this->pt = $pt;
		$this->t = DB_PREFIX.'pt_lexique';
	}
	
	/**
	@function getWord
	
	Rcupration d'un mot
	
	@param	integer	id		identifiant du mot
	@return	object recordset
	*/
	function getWord($id)
	{
		return $this->getWords($id);
	}
	
	/**
	@function getWords
	
	Rcupration d'un ou de plusieurs mots
	
	@param	integer	id		identifiant des mots ('')
	@param	string	letter	lettre des mots ('')
	@param	integer	status	statut des mots (NULL)
	@return	object recordset
	*/
	function getWords($id='',$letter='',$status=NULL,$order_by='D.def_word',$limit='')
	{
		$req_plus = '1';
		
		if ($id != '')
			$req_plus .= ' AND D.def_id='.(integer) $id;
			
		if ($letter != '')
			$req_plus .= ' AND D.def_letter=\''.$letter.'\'';
			
		if ($status !== NULL)
			$req_plus .= ' AND D.def_status='.(integer) $status;
		
		$sql = 
		'SELECT D.def_id, D.def_word, D.def_letter, '.
		'D.def_def, D.def_ex, D.def_user_id, D.def_status, '.
		'U.username AS def_username '.
		'FROM '.$this->t.' D '.
			'LEFT JOIN '.DB_PREFIX.'users U ON D.def_user_id=U.id '.
		'WHERE '.$req_plus.' '.
		'ORDER BY '.$order_by;
				
		if ($limit != '')
			$sql = $sql.' LIMIT '.$limit;
		
		$rs = $this->pt->con->select($sql) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $rs;
	}
		
	/**
	@function setWord
	
	Ajout ou modification d'un mot
	
	@param	string	word		le mot
	@param	string	definition	la dfinition
	@param	string	example		l'exemple ('')
	@param	integer	status		le status du mot (0)
	@param	integer	id			l'identifiant du mot ('')
	@return	boolean
	*/
	function setWord($word,$definition,$example='',$status=0,$id='')
	{
		$definition = $this->pt->linebreaks($definition);
		require_once PT_INC_PATH."libs/punbb_parser.php";
		$definition = preparse_bbcode($definition,false);
		$example = preparse_bbcode($example,false);
		
		if ($example == '')
			$example = NULL;
			
		// update a word
		if ($id != '')
		{
			$updreq = 
			'UPDATE '.$this->t.' SET '.
			'def_word=\''.$this->pt->con->escapeStr($word).'\', '.
			'def_letter=\''.strtoupper($word{0}).'\', '.
			'def_def=\''.$this->pt->con->escapeStr($definition).'\', '.
			'def_ex=\''.$this->pt->con->escapeStr($example).'\', '.
			'def_status='.(integer)$status.' '.
			'WHERE def_id='.$id;
			
			$this->pt->con->execute($updreq) or $this->pt->err->fatalError($this->pt->con->error());
			
			return true;
		}
		// add a word
		else {
			$addreq = 
			'INSERT INTO '.$this->t.' '.
			'(def_word, def_letter, def_def, def_ex, def_user_id, def_status) '.
			'VALUES('.
			'\''.$this->pt->con->escapeStr($word).'\', '.
			'\''.strtoupper($word{0}).'\', '.
			'\''.$this->pt->con->escapeStr($definition).'\', '.
			'\''.$this->pt->con->escapeStr($example).'\', '.
			'\''.$this->pt->user['id'].'\', '.
			''.(integer) $status.')';
			
			$this->pt->con->execute($addreq) or $this->pt->err->fatalError($this->pt->con->error());
			
			return true;
		}
	}
	
	/**
	@function validWord
	
	Validation d'un mot
	
	@param	integer	id			l'identifiant du mot
	@return	boolean
	*/
	function validWord($id)
	{
		if (($mot = $this->getWord($id)) === false)
			return false;

		$updreq = 
		'UPDATE '.$this->t.' SET '.
		'def_status=1 '.
		'WHERE def_id='.$mot->f('def_id');
		
		$this->pt->con->execute($updreq) or $this->pt->err->fatalError($this->pt->con->error());
		
		return true;			
	}
	
	/**
	@function delWord
	
	Suppression d'un mot
	
	@param	integer	id			l'identifiant du mot
	@return	boolean
	*/
	function delWord($id)
	{
		if (($mot = $this->getWord($id)) === false)
			return false;

		$delreq = 
		'DELETE FROM '.$this->t.' '.
		'WHERE def_id='.$mot->f('def_id');
		
		$this->pt->con->execute($delreq) or $this->pt->err->fatalError($this->pt->con->error());
		
		return true;			
	}
	
	/**
	@function getTotalWord
	
	Retourne le nombre total de mots
	
	@return	integer
	*/
	function getTotalWord()
	{
		global $pun_user;
		
		$req = 
		'SELECT COUNT(def_id) AS total_words '.
		'FROM '.$this->t.
		($pun_user['g_id'] > PUN_MOD ? ' WHERE def_status=1' : '');
		
		$result = $this->pt->con->select($req) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $result->f('total_words');		
	}
	
	/**
	@function getTotalValidateWord
	
	Retourne le nombre total de mots  valider
	
	@return	integer
	*/
	function getTotalValidateWord()
	{
		$req = 
		'SELECT COUNT(def_id) AS total_words '.
		'FROM '.$this->t.' WHERE def_status=0';
		
		$result = $this->pt->con->select($req) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $result->f('total_words');		
	}
	
	/**
	@function getTotalAllWord
	
	Retourne le nombre total de mots
	tous statut confondus
	
	@return	integer
	*/
	function getTotalAllWord()
	{
		$req = 'SELECT COUNT(def_id) AS total_words FROM '.$this->t;
		
		$result = $this->pt->con->select($req) or $this->pt->err->fatalError($this->pt->con->error());
				
		return $result->f('total_words');	
	}
    
	/**
	@function searchEntry
	
	Recherche d'un bug selon des mots cl et renvoie un recordset.
	
	@param	string	w			Mots cl
          @param       string          limit                           Limitation du nombre de resultats affichs    
	@return	recordset
	*/
	function searchEntry($w,$limit='')
	{
		$strip = array('+',')','(',',','\'');
		$w = strtolower($w);
		$w = str_replace($strip,' ',$w);
		$w = preg_replace('/( )+/',' ',$w);
		
		$reqPlus = '';
		foreach(explode(' ',$w) as $v)
		{
			$reqPlus .= 
			'AND ( LOWER(def_word) LIKE \'%'.$this->pt->con->escapeStr($v).'%\' OR '.
            ' LOWER(def_ex) LIKE \'%'.$this->pt->con->escapeStr($v).'%\' OR '.
			' LOWER(def_def) LIKE \'%'.$this->pt->con->escapeStr($v).'%\' ) ';
		}
		
		$sql = 'SELECT * FROM '.$this->t. ' WHERE 1=1  '. 	$reqPlus.	'ORDER BY def_word DESC';
        
        if ($limit != '')
			$sql = $sql.' LIMIT '.$limit;     
            		
		$rs = $this->pt->con->select($sql) or $this->pt->err->fatalError($this->pt->con->error());
		
		return $rs;
	}	
}

?>
