<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

require_once PT_INC_PATH."libs/punbb_parser.php";

# Lexique
require dirname(__FILE__).'/inc/lexique.class.php';
$lexique = new lexiqueManager($puntal);

# Template
require dirname(__FILE__).'/inc/tpl_functions.php';
$pt_template_file = '/lexicon/main.php';

# Initialisations
$lexique_mode = 'defaut';

$cur_lettre = '';
$p_mot = '';
$p_def = '';
$p_ex = '';
$edit_id = '';
$del_id = '';
$valide_id = '';

# Titre page
$pt_custom_title_page[] = __('Lexicon');

/**
	Parcours des arguments pour dfinir le mode
	et actions en fonction de ce mode
*/
if (isset($pt_args[1]))
{
	# ajout d'un mot
	if ($pt_args[1] == LEXIQUE_URL_ADD && !$puntal->user['is_guest'])
	{
		$lexique_mode = 'ajouter';
		$pt_custom_title_page[] = __('Add a word');
		
		if (isset($_POST['form_sent']) && $puntal->checkToken())
		{
			$errors = array();
			
			$p_mot = trim($_POST['p_mot']);
			$p_def = trim($_POST['p_def']);
			$p_ex = trim($_POST['p_ex']);
			
			if ($p_mot == '')
				$errors[] = __('You must give a word.');
			
			if ($p_def == '')
				$errors[] = __('You must give a definition.');
			
			if (empty($errors))
			{				
				$p_status = (pt_mod_lexique_validate && $puntal->user['g_id'] > PUN_MOD) ? 0 : 1;

				if ($lexique->setWord($p_mot,$p_def,$p_ex,$p_status))
				{
					if ($p_status == 0)
						$puntal->redirect($puntal->url('lexicon'), __('Word added. It is on standby of validation.'));
					else
						$puntal->redirect($puntal->url('lexicon').strtoupper($p_mot{0}), __('Word added.'));
				}
				else {
					$errors = $puntal->err->getError(false);
				}
			}
		} else $puntal->setToken();
	}
	
	# modification d'un mot
	elseif ($pt_args[1] == LEXIQUE_URL_EDIT && isset($pt_args[2]))
	{
		$lexique_mode = 'modifier';
		$pt_custom_title_page[] = __('Edit a word');
		
		$edit_id = intval($pt_args[2]);
		$mot = $lexique->getWord($edit_id);
		
		if ($puntal->user['g_id'] > PUN_MOD && $puntal->user['id'] != $mot->f('def_user_id'))
			$puntal->err->fatalNotice(__('You do not have permission to access this page.'));
		
		$p_mot = $mot->f('def_word');
		$p_def = $mot->f('def_def');
		$p_ex = $mot->f('def_ex');

		if (isset($_POST['form_sent']))
		{
			$errors = array();
			
			$p_mot = trim($_POST['p_mot']);
			$p_def = trim($_POST['p_def']);
			$p_ex = trim($_POST['p_ex']);
			
			if ($p_mot == '')
				$errors[] = __('You must give a word.');
			
			if ($p_def == '')
				$errors[] = __('You must give a definition.');
			
			if (empty($errors))
			{				
				$p_status = (pt_mod_lexique_validate && $puntal->user['g_id'] > PUN_MOD) ? 0 : 1;
			
				if ($lexique->setWord($p_mot,$p_def,$p_ex,$p_status,$edit_id))
				{				
					if ($p_status == 0)
						$puntal->redirect($puntal->url('lexicon'), __('Word edited. It is on standby of validation.'));
					else
						$puntal->redirect($puntal->url('lexicon').strtoupper($p_mot{0}), __('Word edited.'));
				}
			}
		}
	}
	
	# supression d'un mot
	elseif ($pt_args[1] == LEXIQUE_URL_DEL && isset($pt_args[2]))
	{
		if ($puntal->user['g_id'] > PUN_MOD)
			$puntal->err->fatalNotice(__('You do not have permission to access this page.'));

		$del_id = intval($pt_args[2]);
		$lexique->delWord($del_id);
		
		$puntal->redirect($puntal->url('lexicon'), __('Word deleted.'));
	}
	
	# validation d'un mots
	elseif ($pt_args[1] == LEXIQUE_URL_VALID_WORD && isset($pt_args[2]))
	{
		if ($puntal->user['g_id'] > PUN_MOD)
			$puntal->err->fatalNotice(__('You do not have permission to access this page.'));
		
		$valide_id = intval($pt_args[2]);
		$lexique->validWord($valide_id);
								
		$puntal->redirect($puntal->url('lexicon'), __('Word validated.'));
	}
	
	# affichage de la page de validation
	elseif ($pt_args[1] == LEXIQUE_URL_VALID_PAGE && $puntal->user['g_id'] < PUN_GUEST)
	{
		$lexique_mode = 'valider';
		$words = $lexique->getWords('','',0);
		$num_words = $words->nbRow();
		
		$pt_custom_title_page[] = __('Validating list');
	}
	
	# affichage de la liste complte
	elseif ($pt_args[1] == LEXIQUE_URL_ALL)
	{
		$lexique_mode = 'tous';

		require PT_INC_PATH.'classes/multipage.php';
	
		$nb_mots = $lexique->getTotalWord();
		
		$max_pages = ceil($nb_mots/pt_mod_lexique_num_per_page);
		
		$env = 1;
		if (isset($pt_args[2]) && preg_match('!^page_([0-9]+)!',$pt_args[2],$match))
		{
			$env = $match[1];
		}
		if ($env > $max_pages) $env = 1;
		
		$rs = $lexique->getWords('','',($puntal->user['g_id'] > PUN_MOD ? 1 : NULL),'D.def_word',(($env-1)*pt_mod_lexique_num_per_page).','.pt_mod_lexique_num_per_page);
		
		$lum = new multipage($env, 'allWordsLine', $rs->getData(), $nb_mots, pt_mod_lexique_num_per_page,'','',true);
		
		$lum->setOption('html_block', allWordsBlock());
		
		$lum->setOption('html_row','<tr>%s</tr>');
		$lum->setOption('html_cell','%s');
		
		$lum->setOption('html_links','<p>'.__('Pages').' %s</p>');
		$lum->setOption('html_cur_page','<strong>%s</strong>');
		
		$lum->setOption('html_prev','&lt; '.__('Previous'));
		$lum->setOption('html_next',__('Next').' &gt;');
		$lum->setOption('html_prev_grp','...');
		$lum->setOption('html_next_grp','...');
		$lum->setOption('prefix_url',LEXIQUE_URL_PAGE);

		$lum->setOption('html_empty','<p><strong>'.__('There is no word.').'</strong></p>');
	}
	
	# affichage des mots d'une lettre
	elseif (preg_match('!^([A-Z]$)!', $pt_args[1], $match))
	{
		$lexique_mode = 'lettre';
		$cur_lettre = $pt_args[1];
		$words = $lexique->getWords('',$cur_lettre,($puntal->user['g_id'] > PUN_MOD ? 1 : NULL));
		$num_words = $words->nbRow();
		
		$pt_custom_title_page[] = sprintf(__('Letter %s'),$cur_lettre);
	}
}

?>