<?php
/***********************************************************************

  Copyright (C) 2005-2007 Vincent Garnier and contributors. All rights
  reserved.
  
  This file is part of Puntal 2.

  Puntal 2 is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  Puntal 2 is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/

if (!defined('PUNTAL')) exit;

/**
	La classe de gestion du planet
*/
class planetAdminManager extends planetManager
{
	/**
	@function setFeed
	
	Ajout / modification d'un feed. Renvoie l'id du feed en cas de succs.
	
	@param	string	url			L'URL du feed
	@param	boolean	status		Le status du feed, activ ou dsactiv (true)
	@param	boolean	planet		Ajouter le feed au planet ou pas (true)
	@param	boolean	bloc		Ajouter le feed au bloc ou pas (false)
	@param	integer	id			L'identifiant du feed si modification ('')
	@return	integer
	*/
	function setFeed($url,$status=true,$planet=true,$bloc=false,$id='')
	{
		$url = trim($url);
		
		if ($url == '')
			$this->pt->err->setError(__('You must give a <acronym title="Uniform Resource Locator">URL</acronym> for the feed.'));
		
		if ($this->pt->err->hasError())
			return false;
		
		# Ajout d'un feed
		if ($id == '')
		{			
			$insReq = 
			'INSERT INTO '.$this->table.' '.
			'(rss_url,rss_status,rss_planet,rss_bloc) VALUES ('.
			'\''.$this->pt->con->escapeStr($url).'\', '.
			(integer) ($status ? 1 : 0).', '.
			(integer) ($planet ? 1 : 0).', '.
			(integer) ($bloc ? 1 : 0).
			') ';
			
			$this->pt->con->execute($insReq) or $this->pt->err->fatalError($this->pt->con->error());

			$id = $this->pt->con->getLastID();
			logIt('Planet module : add feed '.$url, 1);
			return $id;
		}
		# Modification d'un feed
		else {
			$updReq = 
			'UPDATE '.$this->table.' SET '.
			'rss_url=\''.$this->pt->con->escapeStr($url).'\', '.
			'rss_status='.(integer) ($status ? 1 : 0).', '.
			'rss_planet='.(integer) ($planet ? 1 : 0).', '.
			'rss_bloc='.(integer) ($bloc ? 1 : 0).' '.
			'WHERE rss_id='.(integer) $id.' ';
			
			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

			logIt('Planet module : update feed '.$url, 1);
			return $id;
		}
	}
		
	/**
	@function delFeed
	
	Suppression d'un feed donn. Renvoie vrai en cas de succs.
	
	@param	integer	id		ID du feed  supprimer
	@return	boolean
	*/
	function delFeed($id)
	{		
		$delReq = 
		'DELETE FROM '.$this->table.' '.
		'WHERE rss_id='.(integer) $id.' ';
		
		$this->pt->con->execute($delReq) or $this->pt->err->fatalError($this->pt->con->error());
		
		return true;
	}
	
	/**
	@function statusFeed
	
	Met un feed en ligne/hors ligne. Renvoie vrai en cas de succs.
	
	@param	integer	id		ID du feed
	@param	integer	status	Nouveau statut (1)
	@return	boolean
	*/
	function statusFeed($id,$status=1)
	{		
		$updReq = 
		'UPDATE '.$this->table.' SET '.
		'rss_status='.(integer) ($status == 1 ? 1 : 0).' '.
		'WHERE rss_id='.(integer) $id.' ';
		
		$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

		return true;
	}
	
	/**
	@function planetFeed
	
	Met un feed sur le planet ou le retirer. Renvoie vrai en cas de succs.
	
	@param	integer	id		ID du feed
	@param	integer	planet	Sur le planet ou non (1)
	@return	boolean
	*/
	function planetFeed($id,$planet=1)
	{		
		$updReq = 
		'UPDATE '.$this->table.' SET '.
		'rss_planet='.(integer) ($planet == 1 ? 1 : 0).' '.
		'WHERE rss_id='.(integer) $id.' ';
		
		$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

		return true;
	}
	
	/**
	@function blockFeed
	
	Met un feed sur le bloc ou le retirer. Renvoie vrai en cas de succs.
	
	@param	integer	id		ID du feed
	@param	integer	block	Sur le bloc ou non (1)
	@return	boolean
	*/
	function blockFeed($id,$block=1)
	{		
		$updReq = 
		'UPDATE '.$this->table.' SET '.
		'rss_bloc='.(integer) ($block == 1 ? 1 : 0).' '.
		'WHERE rss_id='.(integer) $id.' ';
		
		$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());

		return true;
	}
	
	/**
	@function ordFeeds
	
	Ordonne les feeds en utilisant les valeurs du tableau $ord. La cl est
	l'identifiant du feed et la valeur, l'ordre de celui-ci.
	
	La mthode renvoie vrai en cas de succs.
	
	@param	array	ord			Tableau contenant l'ordre de chaque feed
	@return	boolean
	*/
	function ordFeeds($ord)
	{
		if (!is_array($ord))
		{
			$this->pt->err->setError('Bad argument');
			return false;
		}
		
		foreach ($ord as $k => $v)
		{
			$updReq = 
			'UPDATE '.$this->table.' SET '.
			'rss_ord='.(integer) $k.' '.
			'WHERE rss_id='.(integer) $v;
			
			$this->pt->con->execute($updReq) or $this->pt->err->fatalError($this->pt->con->error());
		}
		
		return true;
	}

}

?>